/***************************************************************
 * bst.cpp -- a C++ program to construct and maintain a binary *
 *           search tree annotated with SAIL function calls    *
 ***************************************************************/

#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <assert.h>
#include "bst.h"

#ifdef SWAN
#include "sail.h"
#endif

// new functions added for the annotation purpose
#ifdef SWAN
void rebuild_tree(BST<int>&) ;
bool _tree_insert(BinNode<int>*) ;
void modify_tree(BST<int>&) ;
void _insertnode(BST<int>&) ;
void _deletenode(BST<int>&) ;

// The graph and its layout component
GRAPHID	g = NULLGRAPHID ;	    // graph ID
LC_ID	lc_handle = NULLLCID ;  // Every graph must have a layout component
#endif

// main() is to test whether the binary search tree is correctly 
// constructed and maintained

int GS_MAIN(GS_MAIN_ARGS_F)
{
    BST<int> tree;

#ifdef SWAN
	// Set things up
    sw_init(GS_MAIN_ARGS_A) ;   // Swan initialization
    sw_disablebuttons(RUN) ;    // force viewer step through
    sw_step() ;                 // change mode to STEP
    sw_print("Press STEP to start...") ;
    sw_break() ;
#endif

    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.insert(10);
    tree.print();

    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.remove(10);
    tree.print();

    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.clear();
    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.insert(15);
    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.find(20);
    tree.find(15);
    tree.remove(20);
    tree.insert(20);
    tree.print();

    tree.remove(20);
    tree.print();

    tree.insert(70);
    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.insert(35);
    tree.insert(20);
    tree.insert(17);
    tree.insert(15);
    tree.insert(19);
    tree.insert(100);
    tree.insert(90);
    tree.insert(95);
    tree.insert(1);
    tree.print();

#ifdef SWAN
	// Display the tree, and let the user make changes
    rebuild_tree(tree) ; 
    modify_tree(tree) ;
    sw_break() ;
#endif

    tree.find(100);
    tree.find(99);
    tree.find(20);
    cout << "Need to do some delete tests.\n";
    tree.remove(15);
    tree.print();

    tree.remove(15);
    tree.print();

    tree.clear();
    tree.print();

    cout << "IsEmpty: " << tree.isEmpty() << "\n";

#ifdef SWAN
    sw_quit() ;
#endif

    return(0);
}

#ifdef SWAN
// Clear out the old tree, add the new one, and redraw
void rebuild_tree(BST<int>& tree_ptr)
{
    if (g != NULLGRAPHID) 
        sw_deletegraph(g) ;

    g = sw_newgraph(NULLGRAPHID, BOX, UNDIRECTED) ;
    if (g == NULLGRAPHID) {
        cout << "g cannot be created\n" ;
         exit(0) ;
    }
    sw_setgraphattr(g, GNODECOLOR, DGREEN) ;
    sw_setgraphattr(g, GEDGELABEL, OFF) ;
    lc_handle = sw_newlc(g, NULLLCID, BINTREE) ;
    sw_setcurlc(g, lc_handle) ;
    _tree_insert(tree_ptr.getRoot()) ;
    sw_displayallgraphs() ;
}

// Administer the user's options to modify the tree
void modify_tree(BST<int>& tree)
{
    bool    temp_flag = FALSE ;

    sw_print("Use the EDIT menu to modify the graph, or press STEP to continue...") ;
	// Turn on the menu items to edit
    sw_enablemenuitem(EDITMENU, ITEMINSNODE) ;
    sw_enablemenuitem(EDITMENU, ITEMDELNODE) ;
    while (TRUE) { // Handle the three events that we allow
        switch (sw_wait(STEP|INSNODE|DELNODE)) {
        case STEP:
			sw_print("Press STEP or QUIT");
            temp_flag = TRUE ;
            break ;

        case INSNODE:
            _insertnode(tree) ;
            break ;

        case DELNODE:
            _deletenode(tree) ;
            break ;

        default:
            break ;
        }
        if (temp_flag) break ;
    }
	// Turn off the edit menu items
    sw_disablemenuitem(EDITMENU, ITEMINSNODE) ;
    sw_disablemenuitem(EDITMENU, ITEMDELNODE) ;
}

// Traverse the tree, telling Swan about the nodes and edges
bool _tree_insert(BinNode<int>* node_ptr)
{
    BinNode<int>    *left_child ;
    BinNode<int>    *right_child ;

    if (node_ptr == NULL)
        return TRUE ;
    sw_insertnode(g,(NODEID)node_ptr,inttostr((*node_ptr).value()));

    if ((left_child = (*node_ptr).leftchild()) != NULL) {
        _tree_insert(left_child) ;
        sw_insertbinedge(g, (NODEID)node_ptr, (NODEID)left_child, 
		         NULL, LEFTCHILD) ;
    }
    if ((right_child = (*node_ptr).rightchild()) != NULL) {
        _tree_insert(right_child) ;
        sw_insertbinedge(g, (NODEID)node_ptr, (NODEID)right_child, 
		         NULL, RIGHTCHILD) ;
    }
	return TRUE;	
}

// Handle user input to insert a node (from edit menu)
void _insertnode(BST<int>& tree)
{
    char    temp_str[80] ;
    int	    val ;

    sw_print("Please enter a node value:") ;
    sw_getstr(temp_str) ;
    sscanf(temp_str, "%d", &val) ;
    tree.insert(val) ;
    rebuild_tree(tree) ;
    sprintf(temp_str, "Node %d inserted", val) ;
    sw_print(temp_str) ;
}

// Handle user input to insert a node (from edit menu)
void _deletenode(BST<int>& tree)
{
    char      temp_str[80] ;
    int	      val ;
    NODEID    node_id ;
	char      temp_label[80] ;

    sw_print("Please pick a node:") ;
    sw_picknode(&node_id) ;
    sw_getnodeattr(g, node_id, GNODELABEL, &temp_label) ;
    sscanf(temp_label, "%d", &val) ;
    tree.remove(val) ;
    rebuild_tree(tree) ;
    sprintf(temp_str, "Node %d removed", val) ;
    sw_print(temp_str) ;
}
#endif

/***************************************************************
 * End of bst.cpp                                              *
 ***************************************************************/
