/*******************************************************
heapsort.cpp -- heap-sort an array of integers
In SWAN, show the physical array and logical heap
simultaneously.
*******************************************************/

#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <assert.h>

#include "sail.h"

void rebuild_list(int*, int) ;
void rebuild_heap(int*, int) ;
int	actual_size ;
GRAPHID ga = NULLGRAPHID ; // graph ID of the array
GRAPHID gh = NULLGRAPHID ; // graph ID of the heap
LC_ID	lc_ga = NULLLCID ; // Nominal layout component for array
LC_ID	lc_gh = NULLLCID ; // Nominal layout component for heap

#define ELEM int
#define key(X) (X)

inline void swap(int& a, int& b)
{
  int temp = a;
  a = b;
  b = temp;
}
  
// Heapsort

void siftdown(ELEM* array, int root, int n) {
  char	str1[80] ;
  int	i ;

  while (root<=n/2) {
    int j = 2*root;

    sw_setnodeattr(gh, (NODEID)&array[root], GNODECOLOR, MAGENTA) ;
    sw_setnodeattr(ga, (NODEID)&array[root], GNODECOLOR, MAGENTA) ;
    sprintf(str1, "Comparing node with value %d to its children. Press STEP to continue", 
		array[root]) ;
    sw_displayallgraphs() ;
    sw_print(str1) ;
    sw_break() ;	

    if ((j < n) && (key(array[j]) < key(array[j+1]))) j++;
    if (key(array[root]) >= key(array[j])) {
      sprintf(str1, "Node %d is greater than its children, no swap. Press STEP to continue",
		array[root]) ;
      sw_displayallgraphs() ;
      sw_print(str1) ;
      sw_break() ;	
	  return;
    }

    sprintf(str1, "%d and %d will be swapped. Press STEP to continue", 
		array[root], array[j]) ;
    rebuild_list(array, actual_size) ;
    rebuild_heap(array, n) ;
    sw_setnodeattr(gh, (NODEID)&array[j], GNODEFILLED, TRUE) ;
    sw_setnodeattr(gh, (NODEID)&array[j], GNODECOLOR, LYELLOW) ;
    sw_setnodeattr(gh, (NODEID)&array[root], GNODEFILLED, TRUE) ;
    sw_setnodeattr(gh, (NODEID)&array[root], GNODECOLOR, LYELLOW) ;
    sw_setnodeattr(ga, (NODEID)&array[j], GNODEFILLED, TRUE) ;
    sw_setnodeattr(ga, (NODEID)&array[j], GNODECOLOR, LYELLOW) ;
    sw_setnodeattr(ga, (NODEID)&array[root], GNODEFILLED, TRUE) ;
    sw_setnodeattr(ga, (NODEID)&array[root], GNODECOLOR, LYELLOW) ;
    for (i = n+1 ; i <= actual_size ; i++) {
	  sw_setnodeattr(ga, (NODEID)&array[i], GNODEFILLED, TRUE) ;
      sw_setnodeattr(ga, (NODEID)&array[i], GNODECOLOR, PASGREEN) ;
    }
    sw_displayallgraphs() ;
    sw_print(str1) ;
    sw_break() ;	

    swap(array[root], array[j]);
    rebuild_list(array, actual_size) ;
    rebuild_heap(array, n) ;
    for (i = n+1 ; i <= actual_size ; i++) {
	  sw_setnodeattr(ga, (NODEID)&array[i], GNODEFILLED, TRUE) ;
      sw_setnodeattr(ga, (NODEID)&array[i], GNODECOLOR, PASGREEN) ;
    }
	sw_setnodeattr(gh, (NODEID)&array[j], GNODECOLOR, MAGENTA) ;
    sw_setnodeattr(ga, (NODEID)&array[j], GNODECOLOR, MAGENTA) ;
    sw_displayallgraphs() ;
    sprintf(str1, "Nodes %d and %d are swapped. Press STEP to continue", 
		array[root], array[j]) ;
    sw_print(str1) ;
    sw_break() ;	
	// Now reset the color for next time
    sw_setnodeattr(gh, (NODEID)&array[j], GNODECOLOR, DGREEN) ;
    sw_setnodeattr(ga, (NODEID)&array[j], GNODECOLOR, DGREEN) ;
    root = j;
  }
}

void heapsort(ELEM* array, int n) {
  char	str1[80] ;
  int	i, j;

  for (i=n/2; i>=1; i--) // Build heap
    siftdown(array, i, n);
  sw_print("Heap has been built. Press STEP to sort...") ;
  sw_break() ;

  for (i=n; i>1; i--) { // Sort heap
    sw_setnodeattr(gh, (NODEID)&array[1], GNODECOLOR, MAGENTA) ;
    sw_setnodeattr(ga, (NODEID)&array[1], GNODECOLOR, MAGENTA) ;
    sw_displayallgraphs();
    sprintf(str1, "The next value to be placed is %d.  Press STEP to continue...", array[1]);
    sw_print(str1);
    sw_break();

    sprintf(str1, "%d and %d will be swapped", array[1], array[i]) ;
    sw_setnodeattr(gh, (NODEID)&array[1], GNODEFILLED, TRUE) ;
    sw_setnodeattr(gh, (NODEID)&array[1], GNODECOLOR, LYELLOW) ;
    sw_setnodeattr(gh, (NODEID)&array[i], GNODEFILLED, TRUE) ;
    sw_setnodeattr(gh, (NODEID)&array[i], GNODECOLOR, LYELLOW) ;
    sw_setnodeattr(ga, (NODEID)&array[1], GNODEFILLED, TRUE) ;
    sw_setnodeattr(ga, (NODEID)&array[1], GNODECOLOR, LYELLOW) ;
    sw_setnodeattr(ga, (NODEID)&array[i], GNODEFILLED, TRUE) ;
    sw_setnodeattr(ga, (NODEID)&array[i], GNODECOLOR, LYELLOW) ;
    sw_displayallgraphs() ;
    sw_print(str1) ;
    sw_break() ;

    swap(array[1], array[i]);
    rebuild_list(array, actual_size) ;
    rebuild_heap(array, i-1) ;
    for (j = i ; j <= actual_size ; j++) {
      sw_setnodeattr(ga, (NODEID)&array[j], GNODEFILLED, TRUE) ;
      sw_setnodeattr(ga, (NODEID)&array[j], GNODECOLOR, PASGREEN) ;
    }
    sw_displayallgraphs() ;
    sprintf(str1, "%d is in its final position. Press STEP to continue",
                array[i]) ;
    sw_print(str1) ;
    sw_break() ;

    siftdown(array, 1, i-1);
  }
}

void sort(ELEM* array, int listsize) {
  heapsort(array, listsize);
}

long count1 = 0;
int THRESHOLD = 0;
#define ARRAYSIZE 100
void sort(int* array, int listsize);

void print(int* array, int listsize) {
  for(int i=1; i<=listsize; i++)
    cout << array[i] << " ";
  cout << "\n";
}

int GS_MAIN(GS_MAIN_ARGS_F)
{
  int* array;
  int i;
  int listsize;
  char    temp_str[80] ;

  sw_init(GS_MAIN_ARGS_A) ;
  sw_disablebuttons(RUN) ;
  sw_setcurrentattr(GAUTOREDRAW, FALSE) ;

  sw_print("How many values would you like to sort?") ;
  sw_getstr(temp_str) ;
  sscanf(temp_str, "%d", &listsize) ;

  sw_print("Press STEP to start...") ;
  sw_step() ;
  sw_break() ;

  array = new int[ARRAYSIZE+1];

  actual_size = listsize ;
  if (listsize > ARRAYSIZE) {
    cout << "Selected list size is too big\n";
    exit(-1);
  }
  srand(1);
  for (i=1; i<=ARRAYSIZE; i++)
    array[i] = rand() % 32003;
  for (i=1; i<=listsize; i++)
    if (array[i] < 0) cout << "In INITIALIZE: Neg value!\n";
  print(&array[0], listsize);
  rebuild_list(array, listsize) ;
  rebuild_heap(array, listsize) ;
  sw_displayallgraphs() ;
  sw_print("This is the original array. Press STEP to build the heap...") ;
  sw_break() ;
  sort(&array[0], listsize);
  print(&array[0], listsize);
  rebuild_list(array, listsize) ;
  sw_setgraphattr(ga, GNODECOLOR, MAGENTA) ;
  sw_deletegraph(gh) ;
  sw_displayallgraphs() ;
  sw_print("This is the sorted array. Press STEP to end...") ;
  sw_break() ;
  return(0);
}

void rebuild_list(int* array, int size)
{
  register int	i ;

  if (ga != NULLGRAPHID)
	sw_deletegraph(ga) ;

  ga = sw_newgraph(NULLGRAPHID, BOX, DIRECTED) ;
  if (ga == NULLGRAPHID) {
    cout << "Error: graph cannot be created.\n" ;
	exit(-1) ;
  }
  sw_setgraphattr(ga, GNODEWIDTH, 35) ;
  sw_setgraphattr(ga, GNODEHEIGHT, 16) ;
  lc_ga = sw_newlc(ga, NULLLCID, ARRAYDOWN) ;
  sw_setcurlc(ga, lc_ga) ;

  if (size <= 0) return ;
  for (i = 1 ; i <= size ; i++) 
	sw_insertnode(ga, (NODEID)&array[i], inttostr(array[i]));
}
			 
void rebuild_heap(int* array, int size)
{
  register int	i ;

  if (gh != NULLGRAPHID)
	sw_deletegraph(gh) ;

  gh = sw_newgraph(NULLGRAPHID, BOX, UNDIRECTED) ;
  if (gh == NULLGRAPHID) {
	cout << "Error: graph cannot be created.\n" ;
	exit(-1) ;
  }
  sw_setgraphattr(gh, GNODEWIDTH, 35) ;
  sw_setgraphattr(gh, GNODEHEIGHT, 16) ;
  lc_gh = sw_newlc(gh, NULLLCID, TREE) ;
  sw_setcurlc(gh, lc_gh) ;

  if (size <= 0) return ; 
  sw_insertnode(gh, (NODEID)&array[1], inttostr(array[1])) ;
  for (i = 2 ; i <= size ; i++) {
	sw_insertnode(gh, (NODEID)&array[i], inttostr(array[i]));
	sw_insertedge(gh, (NODEID)&array[i/2], (NODEID)&array[i],
				NULL) ;
  }
}
