#ifndef GRLIST_H
#define GRLIST_H

#include <iostream.h>
#include "stdjeff.h"
#include "stdlib.h"


#define VISITED 1
#define UNVISITED 0

struct EdgeLink {              // A singly linked list node
   int weight;                  // Edge weight
   int v1;                      // Vertex edge comes from
   int v2;                      // Vertex edge goes to
   EdgeLink* next;              // Pointer to next edge in list
   EdgeLink(int vt1, int vt2, int w, EdgeLink* nxt =NULL)
   { v1 = vt1; v2 = vt2; weight = w; next = nxt; }    // Constructor
   EdgeLink(EdgeLink* nxt =NULL) { next = nxt; } // Default Constructor
};
typedef EdgeLink* Edge;

class Graph {                  // Graph class: Adjacency list
protected:
   bool invalid;                // Whether graph was created successfully
   int numVertex;               // Number of vertices
   int numEdge;                 // Number of edges
   Edge* list;                  // The vertex list
   void init() { numVertex=0; numEdge=0; list=NULL; mark=NULL; pred=NULL; }

public:
   bool* mark;                  // The mark array
   int* pred;                   // The predecessor array

   Graph();                     // Constructor
   Graph(istream&);             // Create from a stream
   Graph(const Graph&);         // Copy constructor
   ~Graph();                    // Destructor
   int n();                     // Number of vertices for graph
   int e();                     // Number of edges for graph
   bool isEdge(Edge);           // TRUE if this is an edge
   bool isEdge(int, int);
   Edge first(int);             // Get the first edge for a vertex
   Edge next(Edge);             // Get next edge for a vertex
   Edge getEdge(int, int);
   int v1(Edge);                // Return vertex edge comes from
   int v2(Edge);                // Return vertex edge goes to
   int weight(int, int);        // Return weight of edge
   int weight(Edge);            // Return weight of edge

   bool change_weight(int, int, int);
   virtual bool add_edge(int, int, int);
   virtual bool remove_edge(int, int);
   bool operator !() { return invalid; }
};

#endif

