\begin{verbatim}
/***************************************************************
 *  bst.c -- a C++ program to construct and maintain a  binary *
 *           search tree annotated with SAIL function calls    *
 ***************************************************************/

/* 
   To get the executable file without SWAN annotation:

        g++ -o bst bst.c

   To get the executable file with SWAN annotation:

        g++ -o bst -DSWAN bst.c -L. -lsail++ -lXt -lX11 -lm 

   assuming sail.h and libsail++.a are in current directory.
*/

#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <assert.h>

#ifdef SWAN
#include "sail.h"
#endif

#define FALSE 0
#define TRUE 1
#define DEFAULT_SIZE 10  // size for lists if no size is given.

template <class T> class BinNode {
  public:
    T element;          // The node's value
    BinNode<T>* left;   // Pointer to left child
    BinNode<T>* right;  // Pointer to right child

    BinNode() { left = right = NULL; }
    BinNode(T& e, BinNode<T>* l =NULL, BinNode<T>* r =NULL)
        { element = e; left = l; right = r; }
    ~BinNode() { }
    BinNode<T>* leftchild() { return left; }
    BinNode<T>* rightchild() { return right; }
    T& value() { return element; }
    BinNode<T>* setValue(T& val) { element = val; return this;}
    bool isLeaf() { return (left == NULL) && (right == NULL); }
};

template <class T> class BST {
  private:
    BinNode<T>* root;
    void clearhelp(BinNode<T>*);
    void inserthelp(BinNode<T>*&, T&);
    void removehelp(BinNode<T>*&, T&);
    BinNode<T>* findhelp(BinNode<T>*, T&);
    void printhelp(BinNode<T>*, int) const;
  public:
    BST() { root = NULL; }
    ~BST() { clearhelp(root); }
    BST<T>& clear()
        { clearhelp(root); root = NULL; return *this; }
    BST<T>& insert(T& val)
        { inserthelp(root, val); return *this; }
    BST<T>& remove(T& val)
        { removehelp(root, val); return *this; }
    BinNode<T>* deletemin(BinNode<T>*&);
    BinNode<T>* find(T& val)
        { return findhelp(root, val); }
    bool isEmpty() const
        { return root == NULL; }
    void print() const {
        if (root == NULL) cout << "The BST is empty.\n";
        else printhelp(root, 0);
    }
    BinNode<T>* getRoot(void) { return root ; }
} ;

template <class T>
void BST<T>::clearhelp(BinNode<T>* rt) {
    if (rt == NULL) return;
    clearhelp(rt->leftchild());
    clearhelp(rt->rightchild());
    delete rt;
}

template <class T>
void BST<T>::removehelp(BinNode<T>*& rt, T& val) {
    if (rt == NULL) cout << val << " is not in the tree.\n";
    else if (val < rt->value()) removehelp(rt->left, val);
    else if (val > rt->value()) removehelp(rt->right, val);
    else {  // Found it -- now delete it
        BinNode<T>* temp = rt;
        if (rt->left == NULL) rt = rt->right;
        else if (rt->right == NULL) rt = rt->left;
        else {  // Both children are non-empty
            temp = deletemin(rt->right);
      	    rt->setValue(temp->value());
        }
        delete temp;
    }
}

template <class T>
BinNode<T>* BST<T>::deletemin(BinNode<T>*& rt) {
    BinNode<T>* temp;
    assert(rt != NULL);
    if (rt->left != NULL) return deletemin(rt->left);
    else { temp = rt; rt = rt->right; return temp; }
}

template <class T>
BinNode<T>* BST<T>::findhelp(BinNode<T>* rt, T& val) {
    if (rt == NULL) return NULL;
    else if (val < rt->value())
        return findhelp(rt->leftchild(), val);
    else if (val == rt->value()) return rt;
    else return findhelp(rt->rightchild(), val);
}

template <class T>
void BST<T>::inserthelp(BinNode<T>*& rt, T& val) {
  if (rt == NULL) rt = new BinNode<T>(val, NULL, NULL);
  else if (val < rt->value()) inserthelp(rt->left, val);
  else inserthelp(rt->right, val);
}

template <class T>
void BST<T>::printhelp(BinNode<T> *rt, int level) const {
    int i;
    if (rt == NULL) return;
    printhelp(rt->leftchild(), level+1);
    for (i=0; i<level; i++) cout << "  ";  // indent
        cout << rt->value() << "\n";
    printhelp(rt->rightchild(), level+1);
}

// new functions added for the annotation purpose
#ifdef SWAN
rebuild_tree(BST<int>&) ;
_tree_insert(BinNode<int>*) ;
modify_tree(BST<int>&) ;
_insertnode(BST<int>&) ;
_deletenode(BST<int>&) ;

GRAPHID	g = NULLGRAPHID ;	// graph ID
LCID	lc_handle = NULLLCID ;  // LC ID
#endif

// main() is to test whether the binary search tree is correctly 
// constructed and maintained

int main()
{
    BST<int> tree;

#ifdef SWAN
    sw_init() ;                 // Swan initialization
    sw_disablebuttons(RUN) ;    // force viewer step through
    sw_step() ;                 // change mode to STEP
    sw_print("Press STEP to start...") ;
    sw_break() ;
#endif

    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.insert(10);
    tree.print();

#ifdef SWAN
    rebuild_tree(tree) ; 
    sw_break() ;
#endif

    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.remove(10);
    tree.print();

#ifdef SWAN
    rebuild_tree(tree) ; 
    sw_break() ;
#endif

    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.clear();
    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.insert(15);
    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.find(20);
    tree.find(15);
    tree.remove(20);
    tree.insert(20);
    tree.print();

#ifdef SWAN
    rebuild_tree(tree) ; 
    sw_break() ;
#endif

    tree.remove(20);
    tree.print();

#ifdef SWAN
    rebuild_tree(tree) ; 
    sw_break() ;
#endif

    tree.insert(70);
    cout << "IsEmpty: " << tree.isEmpty() << "\n";
    tree.insert(35);
    tree.insert(20);
    tree.insert(17);
    tree.insert(15);
    tree.insert(19);
    tree.insert(100);
    tree.insert(90);
    tree.insert(95);
    tree.insert(1);
    tree.print();

#ifdef SWAN
    rebuild_tree(tree) ; 
    modify_tree(tree) ;
    sw_break() ;
#endif

    tree.find(100);
    tree.find(99);
    tree.find(20);
    cout << "Need to do some delete tests.\n";
    tree.remove(15);
    tree.print();

#ifdef SWAN
    rebuild_tree(tree) ; 
    sw_break() ;
#endif

    tree.remove(15);
    tree.print();

#ifdef SWAN
    rebuild_tree(tree) ; 
    sw_break() ;
#endif

    tree.clear();
    tree.print();

#ifdef SWAN
    rebuild_tree(tree) ; 
    sw_break() ;
#endif

    cout << "IsEmpty: " << tree.isEmpty() << "\n";

#ifdef SWAN
    sw_quit() ;
#endif

    return(0);
}

#ifdef SWAN
rebuild_tree(BST<int>& tree_ptr)
{
    if (g != NULLGRAPHID) 
        sw_deletegraph(g) ;

    g = sw_newgraph(NULLGRAPHID, BOX, UNDIRECTED) ;
    if (g == NULL) {
        cout << "g cannot be created\n" ;
         exit(0) ;
    }
    sw_setgraphattr(g, GNODECOLOR, DGREEN) ;
    sw_setgraphattr(g, GEDGELABEL, OFF) ;
    lc_handle = sw_newlc(g, NULLLCID, BINTREE) ;
    sw_setcurlc(g, lc_handle) ;
    _tree_insert(tree_ptr.getRoot()) ;
    sw_displayallgraphs() ;
}

modify_tree(BST<int>& tree)
{
    BOOLEAN    temp_flag = FALSE ;

    sw_print("Please modify the graph or STEP to continue...") ;
    sw_enablemenuitem(EDITMENU, ITEMINSNODE) ;
    sw_enablemenuitem(EDITMENU, ITEMDELNODE) ;
    while (TRUE) {
        switch (sw_wait(STEP|INSNODE|DELNODE)) {
        case STEP:
            temp_flag = TRUE ;
            break ;

        case INSNODE:
            _insertnode(tree) ;
            break ;

        case DELNODE:
            _deletenode(tree) ;
            break ;

        default:
            break ;
        }
        if (temp_flag) break ;
    }
    sw_disablemenuitem(EDITMENU, ITEMINSNODE) ;
    sw_disablemenuitem(EDITMENU, ITEMDELNODE) ;
}

_tree_insert(BinNode<int>* node_ptr)
{
    BinNode<int>    *left_child ;
    BinNode<int>    *right_child ;

    if (node_ptr == NULL)
        return TRUE ;
    sw_insertnode(g,(NODEID)node_ptr,inttostr((*node_ptr).value()));

    if ((left_child = (*node_ptr).leftchild()) != NULL) {
        _tree_insert(left_child) ;
        sw_insertbinedge(g, (NODEID)node_ptr, (NODEID)left_child, 
		         NULL, LEFTCHILD) ;
    }
    if ((right_child = (*node_ptr).rightchild()) != NULL) {
        _tree_insert(right_child) ;
        sw_insertbinedge(g, (NODEID)node_ptr, (NODEID)right_child, 
		         NULL, RIGHTCHILD) ;
    }
}

_insertnode(BST<int>& tree)
{
    char    temp_str[80] ;
    int	    val ;

    sw_print("Please enter a node value:") ;
    sw_getstr(temp_str) ;
    sscanf(temp_str, "%d", &val) ;
    tree.insert(val) ;
    rebuild_tree(tree) ;
    sprintf(temp_str, "Node %d inserted", val) ;
    sw_print(temp_str) ;
}

_deletenode(BST<int>& tree)
{
    char      temp_str[80] ;
    LABEL     temp_label ;
    int	      val ;
    NODEID    node_id ;

    sw_print("Please pick a node:") ;
    sw_picknode(&node_id) ;
    sw_getnodeattr(g, node_id, GNODELABEL, &temp_label) ;
    sscanf(temp_label, "%d", &val) ;
    tree.remove(val) ;
    rebuild_tree(tree) ;
    sprintf(temp_str, "Node %d removed", val) ;
    sw_print(temp_str) ;
}
#endif

/***************************************************************
 * End of bst.c                                                *
 ***************************************************************/
\end{verbatim}

