/*****************************************************************
 *           	Minmum Spanning Tree 			 	 *	
 *		      Jun Yang					 *
 *		  October 31, 1993				 *
 *****************************************************************/

/**********
 *	Include files
 **********/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include "mst.h"

/**********
 *	Global variable declarations
 **********/

GRAPHID		g ;
LCID	ld_handle ;
GRAPHID		gc ;
LCID	ld2_handle ;
NODE_LINK 	*g_node_list ; /* head pointer to graph node list */
EDGE_LINK	*g_mst ;       /* head pointer to min_spanning_tree */
WEIGHT		min_weight ;
NODE_ID		min_node1, min_node2 ;

/**********
 *	Main program start here
 **********/

main(int argc, char* argv[])
{
	if (argc != 3)
	    usage() ;

	while (TRUE) {
	    mst(argv[1], argv[2]) ;
	    sw_print("Press RUN/STEP to run again") ;
	    sw_wait(RUN|STEP) ;
	    sw_clear() ;
	    g = gc = NULLGRAPHID ;
	}
}
	
void mst(char* graphname, char* mstname)
{
	NODE_LINK	*cur_node_link ;
	NEIGHBOUR_LINK	*cur_nb_link ;
	WEIGHT		min_cost ;
	NODE_ID		cur_node_id ;
	NODE_LINK	*min_node_link ;
	char		temp_str[128] ;
	int		temp_flag ;

	initialize() ;
	sw_print("Initialized") ;

	if (get_graph(graphname) == -1)
	    error("Input data file error!\n") ;
	create_graph_ll() ;
	sw_displayallgraphs() ;
	sw_print("Please modify the graph or press STEP to continue") ;
	sw_step() ;
	temp_flag = FALSE ;
	sw_enablemenuitem(EDITMENU, ITEMINSNODE) ;
	sw_enablemenuitem(EDITMENU, ITEMDELNODE) ;
	sw_enablemenuitem(EDITMENU, ITEMINSEDGE) ;
	sw_enablemenuitem(EDITMENU, ITEMDELEDGE) ;
	while (TRUE) {
	    switch (sw_wait(STEP|RUN|INSNODE|DELNODE|INSEDGE|DELEDGE)) {
	    case STEP:
	    case RUN:
		temp_flag = TRUE ;
		break ;

	    case INSNODE:
		_insertnode() ;
		break ;

	    case DELNODE:
		_deletenode() ;
		break ;

	    case INSEDGE:
		_insertedge() ;
		break ;

	    case DELEDGE:
		_deleteedge() ;
		break ;
	
	    default:
		break ;
	    }
	    if (temp_flag) break ;
	}
	sw_disablemenuitem(EDITMENU, ITEMINSNODE) ;
	sw_disablemenuitem(EDITMENU, ITEMDELNODE) ;
	sw_disablemenuitem(EDITMENU, ITEMINSEDGE) ;
	sw_disablemenuitem(EDITMENU, ITEMDELEDGE) ;

	if (g_node_list == NULL) 
		error("Empty list!\n");

	/* find a min cost edge (x, y) in G arbitrarily */

	find_min_weight_node(&cur_node_link) ;
	if (cur_node_link == NULL)
	    return ;
	min_node1 = cur_node_link->node_ptr->id ;

	/* mark x TRUE */	
	cur_node_link->node_ptr->mark = TRUE ;

	/* for all edges (x, z), 
	   let z.in_node = x, z.cost = cost(x, z) */

	cur_nb_link = cur_node_link->node_ptr->nb_list ;
	while (cur_nb_link != NULL) {
	    cur_node_id = cur_nb_link->nb_ptr->id ;
	    cur_node_link = g_node_list ;
	    while (cur_node_link != NULL)
		if (cur_node_link->node_ptr->id == cur_node_id) {
		    cur_node_link->node_ptr->cost = 
				cur_nb_link->nb_ptr->weight ;
		    cur_node_link->node_ptr->in_node = min_node1 ;
		    break ;
		}
		else
		    cur_node_link = cur_node_link->next_node ;
	    cur_nb_link = cur_nb_link->next_nb ;
	}

 	while (TRUE) { /* there exists an unmarked vertex */
	    cur_node_link = g_node_list ;
	    while (cur_node_link != NULL) {
		if (cur_node_link->node_ptr->mark == FALSE)
		    break ;
		cur_node_link = cur_node_link->next_node ;
	    }
	    if (cur_node_link == NULL)
		break ; 

	    /* let w be an unmarked vertex such that w.cost is min */
	    min_cost = INFINITELARGE ;
	    cur_node_link = g_node_list ;
	    while (cur_node_link != NULL) {
		if (cur_node_link->node_ptr->mark == FALSE) {
		    if (cur_node_link->node_ptr->cost < min_cost) {
			min_node_link = cur_node_link ;
			min_cost = min_node_link->node_ptr->cost ;
		    }
		}
		cur_node_link = cur_node_link->next_node ;
	    }
	    
	    if (min_node_link->node_ptr->cost == INFINITELARGE)
		error("Graph is not connect!\n") ;
	    else {
		/* mark w TRUE */
		min_node_link->node_ptr->mark = TRUE ;

		/* add (w.in_node, w) to T */
		insert_mst(min_node_link->node_ptr->in_node,
			   min_node_link->node_ptr->id,
			   min_node_link->node_ptr->cost) ;
		sprintf(temp_str, "Edge %d -- %d is inserted in MST",
			min_node_link->node_ptr->in_node,
			min_node_link->node_ptr->id) ;

		sw_setedgeattr(gc,
			(int*)min_node_link->node_ptr->in_node,
			(int*)min_node_link->node_ptr->id,
			GEDGECOLOR, MAGENTA) ;

		sw_setedgeattr(gc,
			(int*)min_node_link->node_ptr->in_node,
			(int*)min_node_link->node_ptr->id,
			GEDGELINETH, THICKLINE) ;
	 
		sw_print(temp_str) ;	
		sw_break() ;
/*
		sw_setedgeattr(gc,
			(int*)min_node_link->node_ptr->id,
			(int*)min_node_link->node_ptr->in_node,
			GEDGECOLOR, MAGENTA) ;
		
		sw_setedgeattr(gc,
			(int*)min_node_link->node_ptr->id,
			(int*)min_node_link->node_ptr->in_node,
			GEDGELINETH, THICKLINE) ;
*/
		/* for all edges (w, z) do */
		cur_nb_link = min_node_link->node_ptr->nb_list ;
		while (cur_nb_link != NULL) {
		    cur_node_id = cur_nb_link->nb_ptr->id ;
		    cur_node_link = g_node_list ;
		    while (cur_node_link != NULL) {
			if (cur_node_link->node_ptr->id == cur_node_id) {
			    if (cur_node_link->node_ptr->mark==FALSE) {
				if (cur_nb_link->nb_ptr->weight <
				    cur_node_link->node_ptr->cost) {
				    cur_node_link->node_ptr->in_node =
					min_node_link->node_ptr->id ;
				    cur_node_link->node_ptr->cost =
					cur_nb_link->nb_ptr->weight ;
				}
			    }
 			    break ;
			}
		        cur_node_link = cur_node_link->next_node ;
		    }
	            cur_nb_link = cur_nb_link->next_nb ;
		}
	    }
	} /* end of main processing loop */

} /* end of main() */

void find_min_weight_node(NODE_LINK** m_node_link)
{
	NODE_LINK	*cur_node_link ;
	NEIGHBOUR_LINK	*cur_nb_link ;
	WEIGHT		min_weight ;

	*m_node_link = NULL ;
	min_weight = INFINITELARGE ;
	cur_node_link = g_node_list ;
	while (cur_node_link != NULL) {
	    cur_nb_link = cur_node_link->node_ptr->nb_list ;
	    while (cur_nb_link != NULL) {
		if (cur_nb_link->nb_ptr->weight < min_weight) {
		    *m_node_link = cur_node_link ;
		    min_weight = cur_nb_link->nb_ptr->weight ;
		}
	        cur_nb_link = cur_nb_link->next_nb ;
	    }
	    cur_node_link = cur_node_link->next_node ;
	}
}

void usage()
{
	fprintf(stdout, "Usage: mst graph_file mst_file\n") ;
	exit(-1) ;
} /* end of usage() */

_insertnode()
{
	int		x, y ;
	int		node1 ;
        NODE_LINK       *cur_ptr, *p ;
	NODE		*cur_node ;
	char		temp_str[80] ;

/*
	sw_disablebuttons(STEP|RUN|INSNODE|DELNODE|INSEDGE|DELEDGE) ;
	sw_print("Please pick a position for the node") ;
	sw_pickpos(&x, &y) ;
*/
	sw_print("Please give an ID for this node:") ;
	sw_getstr(temp_str) ;
	sscanf(temp_str, "%d", &node1) ;
	
/*
	scanf("%d", &node1) ;
	printf("the id of the node is: %d\n", node1) ;
*/
	if (g_node_list != NULL) {
	    p = g_node_list ;
	    while (p != NULL) {
	       if (p->node_ptr->id == node1) {
		   sw_print("Node with the same ID already exists.")
;
		   return ;
	       }
	       p = p->next_node ;
	    }
	}
	
	/* insert this node at the end of the adjacency list */
	cur_node = (NODE *)malloc(sizeof(NODE)) ;
	cur_node->id = node1 ;
	cur_node->cost = INFINITELARGE ;
	cur_node->mark = FALSE ;
	cur_node->nb_list = NULL ;

 	cur_ptr = (NODE_LINK *)malloc(sizeof(NODE_LINK)) ;
	cur_ptr->node_ptr = cur_node ;
	cur_ptr->next_node = NULL ;
	if (g_node_list != NULL) {
	    p = g_node_list ;
	    while (p->next_node != NULL) p=p->next_node ;
	    p->next_node = cur_ptr ;
	}
	else { /* the node list is empty */
	    g_node_list = cur_ptr ;
	}

	sw_setcurlc(gc, ld2_handle) ;
	sw_insertnode(gc, (int*)node1, inttostr(node1)) ;
/*
	sw_setnodeattr(gc, (int*)node1, GNODEPPOS, x, y) ; 
*/
	create_graph_ll() ;
	sw_displayallgraphs() ;
	sprintf(temp_str, "Node %d is inserted", node1) ;
	sw_print(temp_str) ;
/*
	sw_enablebuttons(STEP|RUN|INSNODE|DELNODE|INSEDGE|DELEDGE) ;
*/
}

_deletenode()
{
	int		*node1_d ;
	int		node1 ;
        NODE_LINK       *cur_ptr, *pre_ptr ;
	char		temp_str[80] ;
	GRAPHID		temp_id ;
/*
	sw_disablebuttons(STEP|RUN|INSNODE|DELNODE|INSEDGE|DELEDGE) ;
*/
	sw_print("Please pick a node:") ;
	sw_picknode(&node1_d) ;
	sw_getpickedgraph(&temp_id) ;	
	if (temp_id != gc) {
	    sw_print("Please pick a node in the logical graph!") ;
	    return FALSE ;
	}
	node1 = (int)node1_d ;

	/* search the node in the adjacency list */
	pre_ptr = g_node_list ; 
	if (pre_ptr->node_ptr->id == node1) {
	    g_node_list = pre_ptr->next_node ;
	    del_allnbs(pre_ptr) ;
	    free(pre_ptr) ;
	}
	else {
	    cur_ptr = pre_ptr->next_node ;
	    while (cur_ptr != NULL) {
	        if (cur_ptr->node_ptr->id == node1) {
		    pre_ptr->next_node = cur_ptr->next_node ;
		    del_allnbs(cur_ptr) ;
		    free(cur_ptr) ;
	  	    break ;
	        }
	        else {
		    pre_ptr = cur_ptr ;
		    cur_ptr = cur_ptr->next_node ;
	        }
	    }
	}
	
	cur_ptr = g_node_list ;
	while (cur_ptr != NULL) {
	    del_nb(cur_ptr, node1) ;
	    cur_ptr = cur_ptr->next_node ;
	}

	sw_deletenode(gc, node1_d) ;
	create_graph_ll() ;
	sw_displayallgraphs() ;
	sprintf(temp_str, "Node %d is deleted", node1) ;
	sw_print(temp_str) ;
/*
	sw_enablebuttons(STEP|RUN|INSNODE|DELNODE|INSEDGE|DELEDGE) ;
*/
}

del_nb(NODE_LINK* node_p, int node_d)
{
	NEIGHBOUR_LINK  *cur_link, *pre_link ;

	if (node_p == NULL)
	    return ;

	/* search the edge in the adjacency list */
	pre_link = node_p->node_ptr->nb_list ;
	if (pre_link->nb_ptr->id == node_d) {
	    node_p->node_ptr->nb_list = pre_link->next_nb ;
	    free(pre_link) ;
	    return ;
	}

	cur_link = pre_link->next_nb ;
	while (cur_link != NULL) {
	    if (cur_link->nb_ptr->id == node_d) {
		/* found the neighbour */
		pre_link->next_nb = cur_link->next_nb ;
		free(cur_link) ;
		break ;
	    }
	    else {
		pre_link = cur_link ;
		cur_link = cur_link->next_nb ;
	    }
	}
}

del_allnbs(NODE_LINK* node_p)
{
	NEIGHBOUR_LINK *cur_link, *next_link ;

	next_link = node_p->node_ptr->nb_list ;
	while (next_link != NULL) {
	    cur_link = next_link ;
	    next_link = next_link->next_nb ;
	    free(cur_link) ;
	}
	node_p->node_ptr->nb_list = NULL ;
}

_deleteedge()
{
	int		*node1_d, *node2_d ;
	int		node1, node2 ;
        NODE_LINK       *cur_ptr ;
        NEIGHBOUR_LINK  *cur_link, *pre_link ;
	char		temp_str[80] ;
	GRAPHID		temp_id ;

/*
	sw_disablebuttons(STEP|RUN|INSNODE|DELNODE|INSEDGE|DELEDGE) ;
*/
	sw_print("Please pick an edge:") ;
	sw_pickedge(&node1_d, &node2_d) ;
	sw_getpickedgraph(&temp_id) ;	
	if (temp_id != gc) {
	    sw_print("Please pick an edge in the logical graph!") ;
	    return FALSE ;
	}
	node1 = (int) node1_d ;
	node2 = (int) node2_d ;

	/* search the edge in the adjacency list */
	cur_ptr = g_node_list ;
	while (cur_ptr != NULL) {
	    if (cur_ptr->node_ptr->id == node1) {
		pre_link = cur_ptr->node_ptr->nb_list ;
		if (pre_link->nb_ptr->id == node2) {
		    cur_ptr->node_ptr->nb_list = pre_link->next_nb ;
		    break ;
		}
		cur_link = pre_link->next_nb ;
		while (cur_link != NULL) {
		    if (cur_link->nb_ptr->id == node2) {
			/* found the neighbour */
			pre_link->next_nb = cur_link->next_nb ;
			free(cur_link) ;
			break ;
		    }
		    else {
			pre_link = cur_link ;
			cur_link = cur_link->next_nb ;
		    }
		}
		break ;
	    }
	    else 
		cur_ptr = cur_ptr->next_node ;
	}

	cur_ptr = g_node_list ;
	while (cur_ptr != NULL) {
	    if (cur_ptr->node_ptr->id == node2) {
		pre_link = cur_ptr->node_ptr->nb_list ;
		if (pre_link->nb_ptr->id == node1) {
		    cur_ptr->node_ptr->nb_list = pre_link->next_nb ;
		    break ;
		}
		cur_link = pre_link->next_nb ;
		while (cur_link != NULL) {
		    if (cur_link->nb_ptr->id == node1) {
			/* found the neighbour */
			pre_link->next_nb = cur_link->next_nb ;
			free(cur_link) ;
			break ;
		    }
		    else {
			pre_link = cur_link ;
			cur_link = cur_link->next_nb ;
		    }
		}
		break ;
	    }
	    else 
		cur_ptr = cur_ptr->next_node ;
	}

	sw_deleteedge(gc, node1_d, node2_d) ;
	create_graph_ll() ;
	sw_displayallgraphs() ;
	sprintf(temp_str, "The edge from %d to %d is deleted",
		node1, node2) ;
	sw_print(temp_str) ;
/*
	sw_enablebuttons(STEP|RUN|INSNODE|DELNODE|INSEDGE|DELEDGE) ;
*/
}

_insertedge()
{
	int		*node1_d, *node2_d ;
	int		node1, node2 ;
        NODE_LINK       *cur_ptr ;
        NEIGHBOUR_LINK  *cur_link, *pre_link, *new_link ;
	NEIGHBOUR_NODE	*new_nb ;
	int		temp_weight ;
	char		temp_str[80] ;
	GRAPHID		temp_id ;

/*
	sw_disablebuttons(STEP|RUN|INSNODE|DELNODE|INSEDGE|DELEDGE) ;
*/
	sw_print("Please pick a node:") ;
	sw_picknode(&node1_d) ;
	sw_getpickedgraph(&temp_id) ;	
	if (temp_id != gc) {
	    sw_print("Please pick an edge in the logical graph!") ;
	    return FALSE ;
	}
	sw_print("Please pick another node:") ;
	sw_picknode(&node2_d) ;
	sw_getpickedgraph(&temp_id) ;	
	if (temp_id != gc) {
	    sw_print("Please pick an edge in the logical graph!") ;
	    return FALSE ;
	}
	node1 = (int) node1_d ;
	node2 = (int) node2_d ;
	sw_print("Please give the weight of this edge:") ;
	sw_getstr(temp_str) ;
	sscanf(temp_str, "%d", &temp_weight) ;
	
	/* search the edge in the adjacency list */
	cur_ptr = g_node_list ;
	while (cur_ptr != NULL) {
	    if (cur_ptr->node_ptr->id == node1) {
		pre_link = cur_ptr->node_ptr->nb_list ;
		if (pre_link != NULL) {
		if (pre_link->nb_ptr->id == node2) {
		    sw_print("Edge already exists between the two\
nodes") ; 
		    return FALSE ;
		}
		cur_link = pre_link->next_nb ;
		while (cur_link != NULL) {
		    if (cur_link->nb_ptr->id == node2) {
		        sw_print("Edge already exists between the two\
nodes") ; 
		        return FALSE ;
		    }
		    else {
			pre_link = cur_link ;
			cur_link = cur_link->next_nb ;
		    }
		}
		new_link =
			(NEIGHBOUR_LINK*)malloc(sizeof(NEIGHBOUR_LINK)) ;
		new_nb = (NEIGHBOUR_NODE*)malloc(sizeof(NEIGHBOUR_NODE));
		new_nb->id = node2 ;
		new_nb->weight = temp_weight ;
		new_link->nb_ptr = new_nb ;
		new_link->next_nb = NULL ;
		pre_link->next_nb = new_link ;
		break ;
		}
		else {
		new_link =
			(NEIGHBOUR_LINK*)malloc(sizeof(NEIGHBOUR_LINK)) ;
		new_nb = (NEIGHBOUR_NODE*)malloc(sizeof(NEIGHBOUR_NODE));
		new_nb->id = node2 ;
		new_nb->weight = temp_weight ;
		new_link->nb_ptr = new_nb ;
		new_link->next_nb = NULL ;
		cur_ptr->node_ptr->nb_list = new_link ;
		break ;
	  	}
		
	    }
	    else 
		cur_ptr = cur_ptr->next_node ;
	}

	cur_ptr = g_node_list ;
	while (cur_ptr != NULL) {
	    if (cur_ptr->node_ptr->id == node2) {
		pre_link = cur_ptr->node_ptr->nb_list ;
		if (pre_link != NULL) {
		if (pre_link->nb_ptr->id == node1) {
		    sw_print("Edge already exists between the two\
nodes") ; 
		    return FALSE ;
		}
		cur_link = pre_link->next_nb ;
		while (cur_link != NULL) {
		    if (cur_link->nb_ptr->id == node1) {
		        sw_print("Edge already exists between the two\
nodes") ; 
		        return FALSE ;
		    }
		    else {
			pre_link = cur_link ;
			cur_link = cur_link->next_nb ;
		    }
		}
		new_link =
			(NEIGHBOUR_LINK*)malloc(sizeof(NEIGHBOUR_LINK)) ;
		new_nb = (NEIGHBOUR_NODE*)malloc(sizeof(NEIGHBOUR_NODE));
		new_nb->id = node1 ;
		new_nb->weight = temp_weight ;
		new_link->nb_ptr = new_nb ;
		new_link->next_nb = NULL ;
		pre_link->next_nb = new_link ;
		break ;
		}
		else {
		new_link =
			(NEIGHBOUR_LINK*)malloc(sizeof(NEIGHBOUR_LINK)) ;
		new_nb = (NEIGHBOUR_NODE*)malloc(sizeof(NEIGHBOUR_NODE));
		new_nb->id = node1 ;
		new_nb->weight = temp_weight ;
		new_link->nb_ptr = new_nb ;
		new_link->next_nb = NULL ;
		cur_ptr->node_ptr->nb_list = new_link ;
		break ;
		}
	    }
	    else 
		cur_ptr = cur_ptr->next_node ;
	}

	sw_insertedge(gc, node1_d, node2_d, inttostr(temp_weight)) ;
	create_graph_ll() ;
	sw_displayallgraphs() ;
	sprintf(temp_str, "An edge from %d to %d with weight %d is \
inserted", node1, node2, temp_weight) ;
	sw_print(temp_str) ;
/*
	sw_enablebuttons(STEP|RUN|INSNODE|DELNODE|INSEDGE|DELEDGE) ;
*/
}

void initialize()
{
	min_weight = INFINITELARGE ;
	g_node_list = NULL ;
	g_mst = NULL ;

	sw_init() ;
/*
	g = sw_newgraph(NULLGRAPHID, BOX, DIRECTED) ;
	if (g == NULLGRAPHID ) 
	    error("g cannot be created\n") ;

	sw_setgraphattr(g, GNODECOLOR, DGREEN) ;
	ld_handle = sw_newlc(g, NULLLCID, LISTDOWN) ;
	sw_setlcattr(g, ld_handle, GNODETYPE, TABOXL) ;
	sw_setlcattr(g, ld_handle, GNODEWIDTH, 40) ;
*/
	gc = sw_newgraph(NULLGRAPHID, CIRCLE, UNDIRECTED) ;
	sw_setgraphattr(gc, GNODETYPE, CIRCLE) ;
	sw_setgraphattr(gc, GNODEFILLED, TRUE) ;
	sw_setgraphattr(gc, GNODECOLOR, LYELLOW) ;
	sw_setgraphattr(gc, GNODEWIDTH, 20) ;
	sw_setgraphattr(gc, GNODEHEIGHT, 20) ;
/*
	ld2_handle = sw_newlc(gc, NULLLCID, KKNET) ;
*/
	ld2_handle = sw_newlc(gc, NULLLCID, CIRCLENET) ;
	sw_setcurrentattr(GNODETYPE, BOX) ;
	sw_print("Press RUN/STEP to start") ;
	sw_setcurrentattr(GAUTOREDRAW, FALSE) ;
/*
	sw_invisiblebuttons(INSNODE|DELNODE|INSEDGE|DELEDGE) ;
	sw_disablebuttons(STEP|RUN|INSNODE|DELNODE|INSEDGE|DELEDGE) ;
*/

} /* end of initialize */

int get_graph(char *file_name)
{
	NODE_ID		node1, node2 ;
	WEIGHT		weight ;
	FILE		*file_ptr ;
	int		flag = 0 ;

	if ((file_ptr = fopen(file_name, "rb")) == NULL) 
	    error("Can not open input file !\n") ;

	fscanf(file_ptr, "%d %d %d", &node1, &node2, &weight) ;

	while (!feof(file_ptr)) {
	    insert_edge(node1, node2, weight) ;

	    sw_setcurlc(gc, ld2_handle) ;
	    sw_insertnodeedge(gc, (int*)node1, inttostr(node1),
			      (int*)node2, inttostr(node2), 
			      inttostr(weight)) ;
	
	    if (weight < min_weight) {
		min_weight = weight ;
		min_node1 = node1 ;
		min_node2 = node2 ;
	    }
	    fscanf(file_ptr, "%d %d %d", &node1, &node2, &weight) ;
	}
	
	fclose(file_ptr) ;
	return 0 ;
} /* end of get_graph() */

void insert_edge(NODE_ID node1, NODE_ID node2, WEIGHT weight)
{
	NODE_LINK	*cur_ptr ;
	NODE		*cur_node ;
	BOOLEAN		flag ;
	NODE_LINK	*p;

	cur_ptr = g_node_list ;
	flag = FALSE ;
	while (cur_ptr != NULL) {
	    if (cur_ptr->node_ptr->id == node1) { 
		/* node1 already in the list */
		insert_nb(cur_ptr->node_ptr, node2, weight) ;
		flag = TRUE ;
		break ;
	    }
	    cur_ptr = cur_ptr->next_node ;
	}
	if (!flag) { /* insert node1 to the node_list */
	    cur_node = (NODE *)malloc(sizeof(NODE)) ;
	    cur_node->id = node1 ;
	    cur_node->cost = INFINITELARGE ;
	    cur_node->mark = FALSE ;
	    cur_node->nb_list = NULL ;

/* create a neighbour list */
/*
	    sw_newlc(g, (LCID)&(cur_node->nb_list), LISTACROSS) ;
*/

/* insert this node in the list down graph */
/*
	    sw_setcurlc(g, ld_handle) ;
	    sw_insertnode(g, (int*)cur_node, inttostr(node1)) ;
*/

	    insert_nb(cur_node, node2, weight) ;
 	    cur_ptr = (NODE_LINK *)malloc(sizeof(NODE_LINK)) ;
	    cur_ptr->node_ptr = cur_node ;
	    cur_ptr->next_node = NULL ;
	    if (g_node_list != NULL) {
		p = g_node_list ;
		while (p->next_node != NULL) p=p->next_node ;
		p->next_node = cur_ptr ;
/*
		sw_insertedge(g, (int*)p->node_ptr, (int*)cur_node,
				NULL) ;
*/
	    }
	    else { /* the node list is empty */
 	        cur_ptr = (NODE_LINK *)malloc(sizeof(NODE_LINK)) ;
	        cur_ptr->node_ptr = cur_node ;
 		cur_ptr->next_node = NULL ;
		g_node_list = cur_ptr ;
	    }
	}
	flag = FALSE ;
	cur_ptr = g_node_list ;
	while (cur_ptr != NULL) {
	    if (cur_ptr->node_ptr->id == node2) { 
		/* node2 already in the list */
		insert_nb(cur_ptr->node_ptr, node1, weight) ;
		flag = TRUE ;
		break ;
	    }
	    cur_ptr = cur_ptr->next_node ;
	}
	if (!flag) { /* insert node2 to the node_list */
	    cur_node = (NODE *)malloc(sizeof(NODE)) ;
	    cur_node->id = node2 ;
	    cur_node->cost = INFINITELARGE ;
	    cur_node->mark = FALSE ;
	    cur_node->nb_list = NULL ;

/*
 create a neighbour listE
	    cur_node->lc_handle = sw_newlc(g, LISTACROSS) ;
*/
/*
	    sw_newlc(g, (LCID)&(cur_node->nb_list), LISTACROSS) ;
	  
	    sw_setcurlc(g, ld_handle) ;
	    sw_insertnode(g, (int*)cur_node, inttostr(node2)) ;
*/

	    insert_nb(cur_node, node1, weight) ;
 	    cur_ptr = (NODE_LINK *)malloc(sizeof(NODE_LINK)) ;
	    cur_ptr->node_ptr = cur_node ;
	    cur_ptr->next_node = NULL ;
	    p = g_node_list ;
	    while (p->next_node != NULL) p=p->next_node ;
	    p->next_node = cur_ptr ;

/*
	    sw_insertedge(g, (int*)p->node_ptr, (int*)cur_node, NULL) ;
*/
	}
} /* end of insert_edge() */

void insert_nb(NODE *node_ptr, NODE_ID node1, WEIGHT weight)
{
	NEIGHBOUR_LINK	*cur_link, *p ;
	NEIGHBOUR_NODE  *cur_nb ;
	BOOLEAN		flag ;

	flag = FALSE ;
	cur_link = node_ptr->nb_list ;
	while (cur_link != NULL) {
	    if (cur_link->nb_ptr->id == node1) {
		if (cur_link->nb_ptr->weight != weight)
		    error("Redefinition of edges!\n") ;
		else {
		    flag = TRUE ; /* neighbour node already there */
		    break ;
		}
	    }
	    cur_link = cur_link->next_nb ;
	}
	if (!flag) {
	    cur_nb = (NEIGHBOUR_NODE *)malloc(sizeof(NEIGHBOUR_NODE)) ;
	    cur_nb->id = node1 ;
	    cur_nb->weight = weight ;
	  /*  cur_nb->lw_handle = sw_newlc(g, LISTDOWN) ; */
/*
	    sw_newlc(g, (LCID)&(cur_nb->weight), LISTDOWN) ;

	    sw_setlcattr(g, (LCID)&(cur_nb->weight), GNODETYPE, CIRCLE) ;
	    sw_setlcattr(g, (LCID)&(cur_nb->weight), GNODEFILLED, TRUE) ;
	    sw_setlcattr(g, (LCID)&(cur_nb->weight), GNODECOLOR, LGRAY) ;
	    sw_setlcattr(g, (LCID)&(cur_nb->weight), GNODEWIDTH, 26) ;
	    sw_setlcattr(g, (LCID)&(cur_nb->weight), GNODEHEIGHT, 14) ;
*/
	    cur_link = (NEIGHBOUR_LINK *)
			malloc(sizeof(NEIGHBOUR_LINK)) ;
	    cur_link->nb_ptr = cur_nb ;
/*
	    sw_setcurlc(g, (LCID)&(node_ptr->nb_list)) ;
	    sw_insertnode(g, (int*)cur_nb, inttostr(node1)) ;

	    sw_setcurlc(g, (LCID)&(cur_nb->weight)) ;
	    sw_insertnode(g, (int*)&(cur_nb->weight),
					inttostr(cur_nb->weight)) ;
	    sw_insertedge(g, (int*)cur_nb, (int*)&(cur_nb->weight),
			  NULL) ;
*/
	    if (node_ptr->nb_list != NULL) {
		p = node_ptr->nb_list ;
		while (p->next_nb != NULL) p=p->next_nb ;
/*
		sw_insertedge(g, (int*)p->nb_ptr, 
				    (int*)cur_nb, inttostr(weight)) ;
*/
		p->next_nb = cur_link ;
		cur_link->next_nb = NULL ;
	    }
	    else {
		cur_link->next_nb = NULL ;
		node_ptr->nb_list = cur_link ;
/*
		sw_insertedge(g, (int*)node_ptr, (int*)cur_nb,
				  NULL) ;
*/
	    }
	}
} /* end of insert_nb() */

void insert_mst(NODE_ID node1, NODE_ID node2, WEIGHT weight)
{
	EDGE_LINK	*cur_link ;
	EDGE_LINK	*pre_link ;
	EDGE		*cur_edge ;

	pre_link = NULL ;
	cur_link = g_mst ;
	while(cur_link != NULL) {
	    pre_link = cur_link ;
	    cur_link = cur_link->next_edge ;
	}
	
	cur_edge = (EDGE *) malloc(sizeof(EDGE)) ;
	cur_edge->node1 = node1 ;
	cur_edge->node2 = node2 ;
	cur_edge->weight = weight ;
	
	cur_link = (EDGE_LINK *)malloc(sizeof(EDGE_LINK)) ;
	cur_link->edge_ptr = cur_edge ;
	cur_link->next_edge = NULL ;
	if (pre_link == NULL) 
	    g_mst = cur_link ;
	else 
	    pre_link->next_edge = cur_link ;

} /* end of insert_mst() */

void output_mst(char *file_name)
{
	FILE		*file_ptr ;
	EDGE_LINK	*cur_edge_link ;

	if ((file_ptr = fopen(file_name, "wb")) == NULL)
	    error("Can not open output file!\n") ;

	cur_edge_link = g_mst ;
	while (cur_edge_link != NULL) {
	    fprintf(file_ptr, "%d\t%d\t%d\n", 
		 	cur_edge_link->edge_ptr->node1,
		 	cur_edge_link->edge_ptr->node2,
			cur_edge_link->edge_ptr->weight) ;

/*
            sw_setedgecolor(gc,
                            (int*)(cur_edge_link->edge_ptr->node1),
                            (int*)(cur_edge_link->edge_ptr->node2),
                            LBLUE) ;
            sw_setedgesize(gc,
                            (int*)(cur_edge_link->edge_ptr->node1),
                            (int*)(cur_edge_link->edge_ptr->node2),
                            2) ;
            sw_setedgecolor(gc,
                            (int*)(cur_edge_link->edge_ptr->node2),
                            (int*)(cur_edge_link->edge_ptr->node1),
                            LBLUE) ;
            sw_setedgesize(gc,
                            (int*)(cur_edge_link->edge_ptr->node2),
                            (int*)(cur_edge_link->edge_ptr->node1),
                            2) ;
*/
	    cur_edge_link = cur_edge_link->next_edge ;
	}
	fclose(file_ptr) ;
} /* end of output_mst() */

create_graph_ll()
{
	NODE_LINK	*cur_node_link, *next_node_link ;
	NEIGHBOUR_LINK  *cur_nb_link, *next_nb_link ;

	if (g != NULLGRAPHID) 
	    sw_deletegraph(g) ;
	 
        g = sw_newgraph(NULLGRAPHID, BOX, DIRECTED) ;
        if (g == NULLGRAPHID )
            error("g cannot be created\n") ;

	sw_setgraphattr(g, GNODECOLOR, DGREEN) ;
	sw_setgraphattr(g, GEDGEARROW, ENDARROW) ;
	ld_handle = sw_newlc(g, NULLLCID, LISTDOWN) ;
	sw_setlcattr(g, ld_handle, GNODETYPE, TABOXL) ;
	sw_setlcattr(g, ld_handle, GNODEWIDTH, 40) ;
	cur_node_link = g_node_list ;
	sw_setcurlc(g, ld_handle) ;
	if (cur_node_link != NULL) {
            sw_insertnode(g, (int*)(cur_node_link->node_ptr), 
			  inttostr(cur_node_link->node_ptr->id)) ;
	    sw_newlc(g, (LCID)&(cur_node_link->node_ptr->nb_list),
		     LISTACROSS) ;
	    sw_setcurlc(g, (LCID)&(cur_node_link->node_ptr->nb_list)) ;
	    cur_nb_link = cur_node_link->node_ptr->nb_list ;
	    if (cur_nb_link != NULL) {
		sw_insertnode(g, (int*)(cur_nb_link->nb_ptr), 
				inttostr(cur_nb_link->nb_ptr->id)) ;
		sw_insertedge(g, (int*)(cur_node_link->node_ptr),
			 (int*)(cur_nb_link->nb_ptr), NULL) ;
		next_nb_link = cur_nb_link->next_nb ;
		while (next_nb_link != NULL) {
                    sw_insertnode(g, (int*)(next_nb_link->nb_ptr),
                               inttostr(next_nb_link->nb_ptr->id)) ;
		    sw_insertedge(g, (int*)(cur_nb_link->nb_ptr),
			(int*)(next_nb_link->nb_ptr), NULL) ;
		    cur_nb_link = next_nb_link ;
		    next_nb_link = next_nb_link->next_nb ;
		}
	    }
	    next_node_link = cur_node_link->next_node ;
	    while (next_node_link != NULL) {
		sw_setcurlc(g, ld_handle) ;
                sw_insertnode(g, (int*)(next_node_link->node_ptr), 
			  inttostr(next_node_link->node_ptr->id)) ;
		sw_insertedge(g, (int*)(cur_node_link->node_ptr),
			   (int*)(next_node_link->node_ptr), NULL) ;
		cur_node_link = next_node_link ;
		next_node_link = next_node_link->next_node ;
	        sw_newlc(g, (LCID)&(cur_node_link->node_ptr->nb_list),
		          LISTACROSS) ;
	        sw_setcurlc(g, (LCID)&(cur_node_link->node_ptr->nb_list)) ;
	        cur_nb_link = cur_node_link->node_ptr->nb_list ;
	        if (cur_nb_link != NULL) {
		    sw_insertnode(g, (int*)(cur_nb_link->nb_ptr), 
				inttostr(cur_nb_link->nb_ptr->id)) ;
		    sw_insertedge(g, (int*)(cur_node_link->node_ptr),
			(int*)(cur_nb_link->nb_ptr), NULL) ;
		    next_nb_link = cur_nb_link->next_nb ;
		    while (next_nb_link != NULL) {
                        sw_insertnode(g, (int*)(next_nb_link->nb_ptr),
                               inttostr(next_nb_link->nb_ptr->id)) ;
		        sw_insertedge(g, (int*)(cur_nb_link->nb_ptr),
			(int*)(next_nb_link->nb_ptr), NULL) ;
		        cur_nb_link = next_nb_link ;
		        next_nb_link = next_nb_link->next_nb ;
		    }
	        }
	    } /* end of the downlist */
	}
} /* end of create_graph */

int error(char *error_msg)
{
	fprintf(stderr, error_msg) ;
	exit(-1) ;
} /* end of error() */

/***********************************************************************
 *	End of mst.c						       *
 ***********************************************************************/
