/***********************************************************
 * graph.c --- Member functions of class BasicGraph	   *
 * Jun Yang, November 17, 1994				   *
 ***********************************************************/

/*--------------- Header Files ---------------*/
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
extern "C" {
#include "geosim.h"
#include "gspriv.h"
}
#include "swan.h"
#include "layman.h"
#include "laylist.h"
#include "laycn.h"
#include "laytree.h"
#include "laybtree.h"
#include "laykk.h"
#include "laydir.h"

/*--------------- Member Functions ---------------*/

// constructors and destructors 

BasicGraph::BasicGraph(GRAPHID g_id, NODETYPE n_type)     
{ 
	BasicLayoutComp	*root ;

	id = (g_id == NULLGRAPHID) ? (GRAPHID)this : g_id ;
	node_list = new NODELIST ;
	edge_list = new EDGELIST ;
	lc_list = new LCLIST ; 

	// init root LC and LC list
	root = new BasicLayoutComp(NULLLCID, LISTACROSS) ;
	lc_list->insert(root) ;	
	(*root).setPID(_sw_root_lc) ;
	(*root).setGID(this) ;
	(*(_sw_root_lc->ch_lclist)).insert(root) ;
	cur_lc = root ;

	// set information of the graph
	graph_info = new GRAPHINFO ;
	graph_info->node_type = n_type ; // plain box -- default
	graph_info->directed = _sw_cur_fDirected ; // (un)directed graph 
	graph_info->distX = MINDISTX ; // X-distance -- default
	graph_info->distY = MINDISTY ; // Y-distance -- default
	graph_info->fShowLabel = ON ; // show edge label -- default
	
	// set attributes of nodes in this graph
	node_info = new NODEINFO ;
	switch (graph_info->node_type) {
	    case BOX :	
	    	node_info->size.w = _sw_cur_nodeWidth ;
	    	node_info->size.h = _sw_cur_nodeHeight ;
		break ;
	    case TABOXL :
		node_info->size.w = _sw_cur_nodeWidth ;
		node_info->size.h = _sw_cur_nodeHeight ;
		break ;
	    case CIRCLE:
		node_info->size.w = _sw_cur_nodeWidth ; 
		node_info->size.h = _sw_cur_nodeHeight ;
		break ;
	    default :
		_sw_errno = SWE_INVNODE ;
		_sw_geterrormsg(_sw_errno, _errorstr) ;
		GSelog(_sw_errno, "Swan:%s\n", _errorstr) ;
		break ;
	}
	node_info->offsetX = NODEOFFSETX ;
	node_info->offsetY = NODEOFFSETY ;
	node_info->lineTH = _sw_cur_nodeLineTH ;
	node_info->color = _sw_cur_nodeColor ;
	node_info->filled = _sw_cur_nodeFilled ;

	// set attributes of edges in this graph
	edge_info = new EDGEINFO ;
	edge_info->length = MINEDGELEN ;
	edge_info->line_type = SOLIDLINE ;
	edge_info->lineTH = _sw_cur_edgeLineTH ;
	edge_info->color = _sw_cur_edgeColor ;
	edge_info->arrow_type = graph_info->directed ? ENDARROW : NOARROW ;

	// pass these info to root LC
	*(root->node_info) = *node_info ;
	*(root->edge_info) = *edge_info ;

	// set undisplayed flag
	active = FALSE ;
	nLayout = AUTOLAYOUT ;
	fNewLayout = TRUE ;
	fAutoRelayout = TRUE ;
} 

BasicGraph::BasicGraph(GRAPHID g_id)
{
	BasicLayoutComp	*root = new BasicLayoutComp(NULLLCID, LISTACROSS) ;

	if (g_id == NULLGRAPHID)
	    id = (GRAPHID)this ; // use the graph's address as its ID
	else
	    id = g_id ;		// use the specified ID

	node_list = new NODELIST ;
	edge_list = new EDGELIST ;
	lc_list = new LCLIST ; 

	// init layout components list
	(*lc_list).insert(root) ;	
	(*root).setPID(_sw_root_lc) ;
	(*root).setGID(this) ;
	(*(_sw_root_lc->ch_lclist)).insert(root) ;
	cur_lc = root ;

	// set information of the graph
	graph_info = new GRAPHINFO ;
	graph_info->node_type = _sw_cur_nodeType ; // plain box -- default
	graph_info->directed = _sw_cur_fDirected ; // (un)directed graph 
	graph_info->distX = MINDISTX ; // X-distance -- default
	graph_info->distY = MINDISTY ; // Y-distance -- default
	
	// set attributes of nodes in this graph
	node_info = new NODEINFO ;
	switch (graph_info->node_type) {
	    case BOX :	
	    	node_info->size.w = _sw_cur_nodeWidth ;
	    	node_info->size.h = _sw_cur_nodeHeight ;
		break ;
	    case TABOXL :
		node_info->size.w = _sw_cur_nodeWidth ;
		node_info->size.h = _sw_cur_nodeHeight ;
		break ;
	    case CIRCLE:
		node_info->size.w = _sw_cur_nodeWidth ; 
		node_info->size.h = _sw_cur_nodeHeight ;
		break ;
	    case DABOX:
	    case DDBOX:
	    case TDBOXL:
	    case TABOXC:
	    case TDBOXC:
		_sw_errno = SWE_NOLC ;
		break ;
	    default :
		_sw_errno = SWE_NONODE ;
		break ;
	}
		
	node_info->offsetX = NODEOFFSETX ;
	node_info->offsetY = NODEOFFSETY ;
	node_info->lineTH = _sw_cur_nodeLineTH ;
	node_info->color = _sw_cur_nodeColor ;
	node_info->filled = _sw_cur_nodeFilled ;

	// set attributes of edges in this graph
	edge_info = new EDGEINFO ;
	edge_info->length = MINDISTX ;
	edge_info->line_type = SOLIDLINE ;
	edge_info->lineTH = _sw_cur_edgeLineTH ;
	edge_info->color = _sw_cur_edgeColor ;
	edge_info->arrow_type = graph_info->directed ? ENDARROW : NOARROW ;

	// pass these info to root LC
	*(root->node_info) = *node_info ;
	*(root->edge_info) = *edge_info ;

	// set undisplayed flag
	active = FALSE ;
	nLayout = AUTOLAYOUT ;
	fNewLayout = TRUE ;
	fAutoRelayout = TRUE ;
}

BasicGraph::~BasicGraph()
{
	BasicNode	*p ;
	BasicEdge	*r ;
	BasicLayoutComp	*q ;
	BasicLayoutComp *root = getRootLC() ;
	bool		ok ;

	// clear node list
	ok = (*node_list).getFirstNode(p) ;
	while (ok) {
	    delete p ; 
	    ok = (*node_list).getNextNode(p) ;
	}
	delete node_list ;
	node_list = NULL ;

	// clear edge list
	ok = (*edge_list).getFirstNode(r) ;
	while (ok) {
	    delete r ;
	    ok = (*edge_list).getNextNode(r) ;
	}
	delete edge_list ;
	edge_list = NULL ;

	// remove rootLC from the global LC list
	// delete root ;
	if (_sw_root_lc != NULL) 
	    (*(*_sw_root_lc).getChLCList()).remove(root) ;
	else {
	    _sw_errno = SWE_INTN ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(_sw_errno, "Swan:%s\n", _errorstr) ;
	}

	ok = (*lc_list).getFirstNode(q) ;
	while (ok) {
	    delete q ;
	    ok = (*lc_list).getNextNode(q) ;
	}
	   
	delete lc_list ;
	lc_list = NULL ;

	delete graph_info ;
	delete node_info ;
	delete edge_info ;
	id = NULLGRAPHID ;
	graph_info = NULL ;
	node_info = NULL ;
	edge_info = NULL ;
	cur_lc = NULL ;
}

// search functions

BasicNode* BasicGraph::searchNode(NODEID node_id)
{
	BasicNode	*temp_node ;
	bool		ok ;

	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    if ((*temp_node).getID() == node_id)
	    	return temp_node ;
	    ok = (*node_list).getNextNode(temp_node) ;   
	}
	return NULL ;	
}

BasicEdge* BasicGraph::searchEdge(NODEID src_node, NODEID dest_node)
{
	BasicNode*	from_node ;
	struct _nb_node *temp_nb ;

	if ((from_node = searchNode(src_node)) == NULL) {
	    _sw_errno = SWE_NONODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return NULL ; 
	}
	if ((temp_nb = searchNBNode(from_node, dest_node)) == NULL)  {
	    _sw_errno = SWE_NONBNODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return NULL ; 
	}
	return temp_nb->edge_ptr ;
}

struct _nb_node* searchNBNode(BasicNode* theNode, NODEID nb_node)
{
	NODENBLIST	*temp_nblist = (*theNode).getNBList() ;
	struct _nb_node *temp_nb ;
	bool		ok ;
	
	ok = (*temp_nblist).getFirstNode(temp_nb) ;
	while (ok) {
	    if ((*(temp_nb->id)).getID() == nb_node)
		return temp_nb ;
	    ok = (*temp_nblist).getNextNode(temp_nb) ;
	}
	return NULL ; 
}

BasicEdge* BasicGraph::getEdge(struct _nb_node* nb_node) 
{
	return nb_node->edge_ptr ;
}

bool BasicGraph::insertNBNode(BasicNode* theNode,
                                 struct _nb_node* theNB) 
{
	return (*(theNode->nblist)).insert(theNB) ;
}

bool BasicGraph::removeNBNode(BasicNode* theNode,
                                 struct _nb_node* theNB) 
{
	return (*(theNode->nblist)).remove(theNB) ;
}

/*------------- Basic operations on nodes and edges ------------*
 *			. insertNode				*
 *			. deleteNode				*
 *			. insertEgde				*
 *			. deleteEdge				*
 *--------------------------------------------------------------*/

bool BasicGraph::insertNode(NODEID node_id, LABEL name)
{
	BasicNode 	*q = NULL ;
	BasicNode	*temp_node ;

	if (searchNode(node_id) != NULL) {
	    _sw_errno = SWE_REDEFNODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
  	    return FALSE ;
	}

	// insert this node in the graph's adjacency list
	q = new BasicNode(cur_lc->node_type, node_id, name) ;
	(*q).setAttr(cur_lc->node_info) ;
	q->lc_id = cur_lc ;
	q->nblist = new NODENBLIST ;
	(*node_list).insert(q) ;

	// Check its LC's node list
	// If this is the first node in current LC,
	// set q as the flag node in this LC

	if ((*(cur_lc->ch_nodelist)).getNodeCount() == 0) 
	    cur_lc->fnode = q ;
	else 
	    switch ((*cur_lc).getType()) {
	    case ARRAYDOWN:
	    case ARRAYACROSS:
	    	(*(cur_lc->ch_nodelist)).getLastNode(temp_node) ;
	    	insertEdge((*temp_node).getID(), node_id, NULL) ;
	    	break ;
	    default:
	    	break ;
	    }

	(*(cur_lc->ch_nodelist)).insert(q) ;
	return TRUE ; 
}

bool BasicGraph::deleteNode(NODEID node_id)
{
	BasicNode      	*temp_node ;
	struct _nb_node *temp_nb ;
	NODENBLIST	*temp_list ;
	NODEID		node_id2 ;
	NODELIST	*temp_node_list ;
	BasicLayoutComp *temp_lc, *temp_nb_lc ;
	bool		ok ;

	if ((temp_node = searchNode(node_id)) == NULL) {
	    _sw_errno = SWE_NONODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

	// remove the edges associated with this node
	temp_list = temp_node->nblist ;
	ok = (*temp_list).getFirstNode(temp_nb) ;
	while (ok) {
	    node_id2 = temp_nb->id->id ;
	    deleteEdge(node_id, node_id2) ;
	    ok = (*temp_list).getNextNode(temp_nb) ;	 
	}

	// remove the node from the graph's node list
	(*node_list).remove(temp_node) ;

	// remove the node from it's LC's node list
	if ((temp_lc = (*temp_node).getLayoutComp()) != NULL) {
	    temp_node_list = (*temp_lc).getChNodeList() ;
	    (*temp_node_list).remove(temp_node) ;
	    if ((temp_nb_lc = (*temp_lc).searchNBLC(temp_node)) 
							!= NULL) 
		(*temp_nb_lc).setPNode(NULL) ;
	}

	// delete the node itself
	delete temp_node ;
	return TRUE ;
}	
		
bool BasicGraph::insertEdge(NODEID src_node, NODEID dest_node,
			       LABEL edge_label)
{
	BasicNode	*temp_node1 ;
	BasicNode      	*temp_node2 ;
	struct _nb_node *q = NULL ;
	BasicEdge	*ep = NULL ;
	LAYOUTCOMPID	lc1, lc2, old_pid ;

	if ((temp_node1 = searchNode(src_node)) == NULL) {
	    _sw_errno = SWE_NONODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;	
	}
	if ((temp_node2 = searchNode(dest_node)) == NULL) {
	    _sw_errno = SWE_NONODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;	
	}
	if (searchNBNode(temp_node1, dest_node) != NULL) {
	    _sw_errno = SWE_REDEFEDGE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;	
	}

	q = new (struct _nb_node) ;
	q->id = temp_node2 ;
	ep = new BasicEdge(temp_node1, temp_node2, edge_label) ;

	(*ep).setAttr(edge_info) ;
	(*edge_list).insert(ep) ;
	q->edge_ptr = ep ;
	insertNBNode(temp_node1, q) ;

	if (graph_info->directed == UNDIRECTED) {
	    q = new (struct _nb_node) ;
	    q->id = temp_node1 ;
	    q->edge_ptr = ep ;
	    insertNBNode(temp_node2, q) ;
	}

	// set edge's LC
        lc1 = (*temp_node1).getLayoutComp() ;
        lc2 = (*temp_node2).getLayoutComp() ;
	if (lc1 == lc2) 
	    (*ep).setLayoutComp(lc1) ;
	else {	
	    if (temp_node2 == lc2->fnode &&
		temp_node2->lc_id->pnode == NULL) {
		lc2->pnode = temp_node1 ;
		old_pid = (*lc2).getPID() ;
		(*(old_pid->ch_lclist)).remove(lc2) ;
		(*lc2).setPID(lc1) ;		
		(*(lc1->ch_lclist)).insert(lc2) ;
	    	(*ep).setLayoutComp(lc1) ;
	    }
	    else 
		(*ep).setLayoutComp(findCCA(lc1, lc2)) ;
	}
	return TRUE ;
}

bool BasicGraph::deleteEdge(NODEID src_node, NODEID dest_node)
{
	BasicNode	*temp_node1, *temp_node2 ;
	struct _nb_node *temp_nb ;
	BasicEdge	*temp_edge ;

	if ((temp_node1 = searchNode(src_node)) == NULL) {
	    _sw_errno = SWE_NONODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;	
	}
	if ((temp_node2 = searchNode(dest_node)) == NULL) {
	    _sw_errno = SWE_NONODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	if ((temp_nb = searchNBNode(temp_node1, dest_node)) == NULL) {
	    _sw_errno = SWE_NONBNODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;	
	}
	if ((temp_edge = getEdge(temp_nb)) == NULL) {
	    _sw_errno = SWE_NOEDGE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;	
	}
	removeNBNode(temp_node1, temp_nb) ;	
	if (graph_info->directed == UNDIRECTED) {
	    if ((temp_nb = searchNBNode(temp_node2, src_node)) ==
							NULL) {
	        _sw_errno = SWE_INTN ;  
	        _sw_geterrormsg(_sw_errno, _errorstr) ;
	        GSelog(_sw_errno, "Swan:%s\n", _errorstr) ;
	    }
	    removeNBNode(temp_node2, temp_nb) ;
	}
	(*edge_list).remove(temp_edge) ;
	delete temp_edge ;
	return TRUE ;	
}

// graph access member functions

bool BasicGraph::setDirected(FLAG d_flag)
{
	graph_info->directed = d_flag ;
	edge_info->arrow_type = d_flag ? ENDARROW : NOARROW ;
	return TRUE ;
}

bool BasicGraph::isDirected(void) 
{
	if (graph_info->directed == DIRECTED)
	    return TRUE ;
	return FALSE ;
}

bool BasicGraph::setGraphInfo(GRAPHINFO* g_info)
{
	*graph_info = *g_info ;
	return TRUE ;
}

GRAPHINFO* BasicGraph::getGraphInfo(void)
{
	return graph_info ;
}

bool BasicGraph::setNodeInfo(NODEINFO* n_info)
{
	*node_info = *n_info ;
	return TRUE ;
}

NODEINFO* BasicGraph::getNodeInfo(void)
{
	return node_info ;
}

bool BasicGraph::setEdgeInfo(EDGEINFO* e_info)
{
	*edge_info = *e_info ;
	return TRUE ;
}

EDGEINFO* BasicGraph::getEdgeInfo(void)
{
	return edge_info ;
}

bool BasicGraph::setNodeType(NODETYPE n_type)
{
	BasicNode*	temp_node ;
	bool		ok ;

	graph_info->node_type = n_type ;            
	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).setType(n_type) ;
	    ok = (*node_list).getNextNode(temp_node) ;
	}
	return TRUE ;
}

NODETYPE BasicGraph::getNodeType(void)
{
	return graph_info->node_type ;            
}

bool BasicGraph::setNodeColor(COLORTYPE n_color)
{
	BasicNode*	temp_node ;
	bool		ok ;

	node_info->color = n_color ;            
	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).setColor(n_color) ;
	    ok = (*node_list).getNextNode(temp_node) ;
	}
	return TRUE ;
}

COLORTYPE BasicGraph::getNodeColor(void)
{
	return node_info->color ;            
}

bool BasicGraph::setNodeFilled(FLAG f_flag) 
{
	BasicNode*	temp_node ;
	bool		ok ;

	node_info->filled = f_flag ;            
	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).setFilled(f_flag) ;
	    ok = (*node_list).getNextNode(temp_node) ;
	}
	return TRUE ;
}

FLAG BasicGraph::getNodeFilled(void) 
{
	return node_info->filled ;
}

bool BasicGraph::setNodeWidth(int width) 
{
	BasicNode*	temp_node ;
	bool		ok ;

	node_info->size.w = width ;            
	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).setWidth(width) ;
	    ok = (*node_list).getNextNode(temp_node) ;
	}
	return TRUE ;
}

int BasicGraph::getNodeWidth(void) 
{
	return node_info->size.w ;
}

bool BasicGraph::setNodeHeight(int height) 
{
	BasicNode*	temp_node ;
	bool		ok ;

	node_info->size.h = height ;            
	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).setHeight(height) ;
	    ok = (*node_list).getNextNode(temp_node) ;
	}
	return TRUE ;
}

int BasicGraph::getNodeHeight(void) 
{
	return node_info->size.h ;
}

bool BasicGraph::setNodeLineTH(int line_TH) 
{
	BasicNode*	temp_node ;
	bool		ok ;

	node_info->lineTH = line_TH ;            
	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).setLineTH(line_TH) ;
	    ok = (*node_list).getNextNode(temp_node) ;
	}
	return TRUE ;
}

int BasicGraph::getNodeLineTH(void) 
{
	return node_info->lineTH ;
}

bool BasicGraph::setEdgeLength(int e_length) 
{
	BasicEdge*	temp_edge ;
	bool		ok ;

	edge_info->length = e_length ;            
	ok = (*edge_list).getFirstNode(temp_edge) ;
	while (ok) {
	    (*temp_edge).setLength(e_length) ;
	    ok = (*edge_list).getNextNode(temp_edge) ;
	}
	return TRUE ;
}

int BasicGraph::getEdgeLength(void) 
{
	return edge_info->length ;
}

bool BasicGraph::setEdgeLineTH(int line_TH) 
{
	BasicEdge*	temp_edge ;
	bool		ok ;

	edge_info->lineTH = line_TH ;            
	ok = (*edge_list).getFirstNode(temp_edge) ;
	while (ok) {
	    (*temp_edge).setLineTH(line_TH) ;
	    ok = (*edge_list).getNextNode(temp_edge) ;
	}
	return TRUE ;
}

int BasicGraph::getEdgeLineTH(void) 
{
	return edge_info->lineTH ;
}

bool BasicGraph::setEdgeArrow(ARROWTYPE e_arrow) 
{
	BasicEdge*	temp_edge ;
	bool		ok ;

	edge_info->arrow_type = e_arrow ;            
	ok = (*edge_list).getFirstNode(temp_edge) ;
	while (ok) {
	    (*temp_edge).setArrow(e_arrow) ;
	    ok = (*edge_list).getNextNode(temp_edge) ;
	}
	return TRUE ;
}

bool BasicGraph::setEdgeColor(COLORTYPE e_color) 
{
	BasicEdge*	temp_edge ;
	bool		ok ;

	edge_info->color = e_color ;            
	ok = (*edge_list).getFirstNode(temp_edge) ;
	while (ok) {
	    (*temp_edge).setColor(e_color) ;
	    ok = (*edge_list).getNextNode(temp_edge) ;
	}
	return TRUE ;
}

COLORTYPE BasicGraph::getEdgeColor(void) 
{
	return edge_info->color ;            
}

// functions related to graph display
	
bool BasicGraph::setAllNodesPPos(void)
{
	BasicNode*	temp_node ;
	LAYOUTCOMPID	temp_lc ;
	LOCATION	temp_pos1, temp_pos2 ;
	bool		ok ;

	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    temp_pos1 = (*temp_node).getRPos() ;
	    if (temp_pos1 != UnsetPos) {
	        temp_lc = (*temp_node).getLayoutComp() ;
	        temp_pos2 = (*temp_lc).getPPos() ;
	        temp_pos1.x += temp_pos2.x ;
	        temp_pos1.y += temp_pos2.y ;
	        (*temp_node).setPPos(temp_pos1) ;
	        (*temp_node).disActive() ; 			// ??
	    }
	    ok = (*node_list).getNextNode(temp_node) ;
	}
	return TRUE ;
}

bool BasicGraph::edgeIn(int x, int y)
{
	BasicEdge	*temp_edge ;
	bool		ok ;

	ok = (*edge_list).getFirstNode(temp_edge) ;
	while (ok) {
	    if ((*temp_edge).nearThisEdge(x, y))
	 	return TRUE ;
	    ok = (*edge_list).getNextNode(temp_edge) ;
	}
	return FALSE ;
}

// Input/Output Functions 
//	. save
//	. load	

bool BasicGraph::save(ofstream& gout)
{
	BasicEdge*	temp_edge ;
	bool		ok ;

	if (!gout) 
	    return FALSE ;
	
	gout << '(' << graph_info->node_type << ' ' << 
			graph_info->directed << ' ' <<
			graph_info->fShowLabel << ')' << '\n' ;
	gout << '(' << node_info->shape << ' ' << node_info->size.w <<
		' ' << node_info->size.h << ' ' << 
		' ' << node_info->color << ' ' << node_info->filled <<
		' ' << node_info->lineTH << 
		' ' << (*node_list).getNodeCount() << ')' << '\n' ;
	gout << '(' << edge_info->length << ' ' << edge_info->lineTH 
	     <<	' ' << edge_info->color << ' ' << edge_info->label_ppos.x
	     <<	' ' << edge_info->label_ppos.y << ' '
 	     << (*edge_list).getNodeCount() << ')' << '\n' ;

	if (!(*getRootLC()).save(gout, 0))
	    return FALSE ; 

	ok = (*edge_list).getFirstNode(temp_edge) ;
	while (ok) {
	    if (!(*temp_edge).save(gout))
		return FALSE ; 
	    ok = (*edge_list).getNextNode(temp_edge) ;
	}
	return TRUE ;
}

bool BasicGraph::load(ifstream& gin)
{
	char		c ;
	int		num_node, num_edge ;
	BasicEdge 	*temp_edge ;
	BasicNode 	*node1, *node2 ;
	struct _nb_node	*q ;
	
	if (!gin) 
	    return FALSE ;
	// load graph info
	gin >> c ;
	if (c != '(') 
	    return FALSE ;
	gin >> graph_info->node_type >> graph_info->directed >> 
		graph_info->fShowLabel >> c ;
	if (c != ')')
	    return FALSE ;

	// load node info
	gin >> c ;
	if (c != '(') 
	    return FALSE ;
	gin >> node_info->shape >> node_info->size.w >>
	       node_info->size.h >>  
	       node_info->color >> node_info->filled >>
	       node_info->lineTH >> num_node >> c ;
	if (c != ')')
	    return FALSE ;

	// load edge info
	gin >> c ;
	if (c != '(') 
	    return FALSE ;
	gin >> edge_info->length >> edge_info->lineTH 
	    >> edge_info->color >> edge_info->label_ppos.x 
	    >> edge_info->label_ppos.y >> num_edge >> c ;
	if (c != ')')
	    return FALSE ;

	// load LC and nodes
	(*getRootLC()).load(gin, 0, _sw_root_lc) ;
	if (num_node != (*node_list).getNodeCount())
	    return FALSE ;

	// load edges
	while (TRUE) {
	    temp_edge = new BasicEdge ;
	    if (!(*temp_edge).load(gin, this))
		return FALSE ;
	    (*edge_list).insert(temp_edge) ;

	    // add in the neighbouring node in NODELIST
	    node1 = (*temp_edge).getSNode() ;
	    node2 = (*temp_edge).getENode() ;
	    q = new _nb_node ;
	    q->id = node2 ;
	    q->edge_ptr = temp_edge ;
	    insertNBNode(node1, q) ;

	    if (graph_info->directed == UNDIRECTED) {
	        q = new _nb_node ;
	        q->id = node1 ;
	        q->edge_ptr = temp_edge ;
	    	insertNBNode(node2, q) ;
	    }
	
	    gin >> c ;
	    if (gin.eof()) 
		break ;
	    else 
		gin.putback(c) ;
	}


	if (num_edge != (*edge_list).getNodeCount())
	    return FALSE ;

	(*getRootLC()).restorePFNode() ;
	// if we comes here, the graph is successfully loaded
	return TRUE ;
}

GRAPHID BasicGraph::getID(void)
{
	return id ;
}

NODELIST* BasicGraph::getNodeList()
{
	return node_list ;
}

LCLIST* BasicGraph::getLCList()
{
	return lc_list ;
}

bool BasicGraph::isDisplayed(void)
{
	return active ;
}

FLAG BasicGraph::isNewLayout(void)
{
	return fNewLayout ;
}

void BasicGraph::setNewLayoutFlag(FLAG f_new)
{
	fNewLayout = f_new ;
}

FLAG BasicGraph::isAutoRelayout(void)
{
	return fAutoRelayout ;
}

void BasicGraph::setAutoRelayout(FLAG f_new)
{
	fAutoRelayout = f_new ;
}

void BasicGraph::enActive(void)
{
	active = TRUE ;
}

void BasicGraph::disActive(void)
{
	active = FALSE ;
}

void BasicGraph::setTravFlags(void)
{
	bool		ok ;
	BasicNode	*temp_node ;

	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).setTravFlag() ;
	    ok = (*node_list).getNextNode(temp_node) ;
	}
}

void BasicGraph::clearTravFlags(void)
{
	bool		ok ;
	BasicNode	*temp_node ;

	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).clearTravFlag() ;
	    ok = (*node_list).getNextNode(temp_node) ;
	}
}

EDGELIST* BasicGraph::getEdgeList(void)
{
	return edge_list ;
}

LAYOUTCOMPTYPE BasicGraph::getLayoutType(void)
{
	BasicLayoutComp		*temp_lc ;
	LAYOUTCOMPTYPE		temp_type ;

	(*(getRootLC()->ch_lclist)).getFirstNode(temp_lc) ;
	temp_type = (*temp_lc).getType() ;
	switch (temp_type) {
	case LISTACROSS:
	case LISTDOWN:
	    return LISTACROSS;

	case ARRAYDOWN:
	case ARRAYACROSS:
	    return ARRAYDOWN ;

	default:
	    return temp_type ;
	}
}

bool BasicGraph::setLayoutType(LAYOUTCOMPTYPE lc_type)
{
	BasicLayoutComp		*temp_lc ;

	(*(getRootLC()->ch_lclist)).getFirstNode(temp_lc) ;
	return (*temp_lc).setType(lc_type) ;
}

int BasicGraph::getLayoutMode(void)
{
	return nLayout ;
}

bool BasicGraph::setLayoutMode(int newlayoutmethod)
{
	if (newlayoutmethod == MANULAYOUT ||
	    newlayoutmethod == AUTOLAYOUT) {
	    nLayout = newlayoutmethod ;
	    return TRUE ;
	}
	return FALSE ;
}

LOCATION BasicGraph::getPPos(void)
{
	return getRootLC()->getPPos() ;
}

bool BasicGraph::setPPos(LOCATION g_pos)
{
	return getRootLC()->setPPos(g_pos) ;
}

// Graph Display Operations
//	.layoutGraph
// 	.displayGraph

bool BasicGraph::layoutGraph(void)
{
	LAYOUTCOMPID	root_lc ;
	BasicLayoutComp *temp_lc ;

	root_lc = getRootLC() ;			

	// some work need to be done here 
        if ((*(root_lc->ch_lclist)).getFirstNode(temp_lc)) 
	    switch (temp_lc->type) {
	    case MANUAL:
		Manual_layout(this) ;
		break ;

	    case LISTACROSS:
	    case LISTDOWN:
	    case ARRAYACROSS:
	    case ARRAYDOWN:
		List_layout(this) ;
		break ;

	    case CIRCLENET:
		CN_layout(this) ;
		break ;

	    case TREE:
		Tree_layout(this) ;
		break ;

	    case BINTREE:
		BinTree_layout(this) ;
		break ;

	    case KKNET:
	        KK_layout(this) ;
		break ;
	
	    case HIERACHY:
		DI_layout(this) ;
		break ;
	    
	    default:
		_sw_errno = SWE_NOLC ;
		_sw_geterrormsg(_sw_errno, _errorstr) ;
		GSelog(0, "Swan:%s\n", _errorstr) ;
		return FALSE ;
	    }
	return TRUE ;
}

bool BasicGraph::displayGraph(void)
{
	LAYOUTCOMPID	root ;
	BasicLayoutComp *temp_lc ;
	BasicNode	*temp_node ;
	BasicEdge	*temp_edge ;
	bool		ok ;

	// set positions of nodes
	root = getRootLC() ;
	(*root).setChLCPPos() ;
	setAllNodesPPos() ;
	
	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	GSdoeventsandreturn() ;

	// display all the nodes
	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).displayNode() ;
	    ok = (*node_list).getNextNode(temp_node) ;    
	}

	// some work need to be done here
	if ((*(root->ch_lclist)).getFirstNode(temp_lc)) 
	    switch (temp_lc->type) {
	    case MANUAL:
		_Manual_edge_ppath(this) ;
		break ;

	    case LISTDOWN:
	    case LISTACROSS:
	    case ARRAYDOWN:
	    case ARRAYACROSS:
		_List_edge_ppath(this) ;
		break ;

	    case CIRCLENET:
		_CN_edge_ppath(this) ;
		break ;

	    case KKNET:
		_KK_edge_ppath(this) ;
		break ;

	    case HIERACHY:
		_DIGRAPH_edge_ppath(this) ;
		break ;

	    case TREE:
	        if (graph_info->fShowLabel) {
		    clearTravFlags() ;
		    _TREE_labeled_edge_ppath(temp_lc->fnode) ;
		}
		else 
		    _TREE_nolabel_edge_ppath(this) ;
		break ;

	    case BINTREE:
	        if (graph_info->fShowLabel) {
		    clearTravFlags() ;
		    _BINTREE_labeled_edge_ppath(temp_lc->fnode) ;
		}
		else 
		    _BINTREE_nolabel_edge_ppath(this) ;
		break ;

	    default:
		_sw_errno = SWE_NOLC ;
		_sw_geterrormsg(_sw_errno, _errorstr) ;
		GSelog(0, "Swan:%s\n", _errorstr) ;
		return FALSE ;
	    }

	// display all the edges
	ok = (*edge_list).getFirstNode(temp_edge) ;
	while (ok) {
	    (*temp_edge).displayEdge() ;
	    ok = (*edge_list).getNextNode(temp_edge) ;    
	}
	active = TRUE ;
	// GSPflush() ;
	GSdoeventsandreturn() ;
	return TRUE ;
}

bool BasicGraph::dump(void)
{
	BasicNode	*temp_node ;
	BasicEdge	*temp_edge ;
	bool		ok ;

	if (!isDisplayed()) 
	    return FALSE ;

	GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
	GSdoeventsandreturn() ;
	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).dump() ;
	    ok = (*node_list).getNextNode(temp_node) ;    
	}
	ok = (*edge_list).getFirstNode(temp_edge) ;
	while (ok) {
	    (*temp_edge).dump() ;
	    ok = (*edge_list).getNextNode(temp_edge) ;    
	}
	GSdoeventsandreturn() ;
	return TRUE ;
}

bool BasicGraph::pan(int xoffset, int yoffset)
{
	BasicNode	*temp_node ;
	BasicEdge	*temp_edge ;
	bool		ok ;

	if (!isDisplayed()) 
	    return FALSE ;

	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).pan(xoffset, yoffset) ;
	    ok = (*node_list).getNextNode(temp_node) ;    
	}
	ok = (*edge_list).getFirstNode(temp_edge) ;
	while (ok) {
	    (*temp_edge).pan(xoffset, yoffset) ;
	    ok = (*edge_list).getNextNode(temp_edge) ;
	}
	return TRUE ;
}
	
bool BasicGraph::zoom(void)
{
	BasicNode	*temp_node ;
	BasicEdge	*temp_edge ;
	bool		ok ;

	if (!isDisplayed()) 
	    return FALSE ;

	ok = (*node_list).getFirstNode(temp_node) ;
	while (ok) {
	    (*temp_node).zoom() ;
	    ok = (*node_list).getNextNode(temp_node) ;    
	}
	ok = (*edge_list).getFirstNode(temp_edge) ;
	while (ok) {
	    (*temp_edge).zoom() ;
	    ok = (*edge_list).getNextNode(temp_edge) ;
	}
	return TRUE ;
}
	
// Operations on Layout Components 

LC_ID BasicGraph::createLC(LC_ID lc_id, LAYOUTCOMPTYPE lc_type) 
{
	BasicLayoutComp *p = NULL ;
	BasicLayoutComp *root = getRootLC() ;

	if (lc_id != NULLLCID) {
	    if (searchLC(lc_id) != NULL) {
		// LC with the same ID already existed
		_sw_errno = SWE_BADLCID ;
		_sw_geterrormsg(_sw_errno, _errorstr) ;
		GSelog(0, "Swan:%s\n", _errorstr) ;
	 	return NULLLCID ;
	    }
	    p = new BasicLayoutComp(lc_id, lc_type) ;

 	    // suspicious: 
	    (*lc_list).insert(p) ;

	    (*p).setPID(root) ;
	    (*p).setGID(this) ;
	    (*(root->ch_lclist)).insert(p) ;
	    (*p).setNodeType(graph_info->node_type) ;
	    (*p).setNodeInfo(node_info) ;
	    (*p).setEdgeInfo(edge_info) ;
	    return lc_id ;
	}
	else {
	    p = new BasicLayoutComp(NULLLCID, lc_type) ;

	    // suspicious
	    (*lc_list).insert(p) ;

	    (*p).setPID(root) ;
	    (*p).setGID(this) ;
	    (*(root->ch_lclist)).insert(p) ;
	    (*p).setNodeType(graph_info->node_type) ;
	    (*p).setNodeInfo(node_info) ;
	    (*p).setEdgeInfo(edge_info) ;
	    return (*p).getEID() ;
        }
}

bool BasicGraph::removeLC(LC_ID lc_id) 
{
	BasicLayoutComp*	lc_ptr = NULL ;

	if ((lc_ptr = searchLC(lc_id)) == NULL) {
	    _sw_errno = SWE_NOLC ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	delete lc_ptr ;
	(*lc_list).remove(lc_ptr) ;
	return TRUE ;
}

BasicLayoutComp* BasicGraph::searchLC(LC_ID lc_id)
{
	BasicLayoutComp*	lc_ptr = NULL ;
	bool			ok ;

	ok = (*lc_list).getFirstNode(lc_ptr) ;
	while (ok) {
	    if ((*lc_ptr).getEID() == lc_id) 
		return lc_ptr ;
	    ok = (*lc_list).getNextNode(lc_ptr) ;
	}
	return NULL ;
}

void BasicGraph::setFlagNode(LAYOUTCOMPID lc_id, INNODEID innode_id)
{
	lc_id->fnode = innode_id ;
}

bool BasicGraph::setCurLC(LC_ID lc_id)
{
	BasicLayoutComp*	lc_ptr = NULL ;

	if ((lc_ptr = searchLC(lc_id)) == NULL) {
	    _sw_errno = SWE_NOLC ;
	    return FALSE ;
	}
	cur_lc = lc_ptr ;
	return TRUE ;
}

LAYOUTCOMPID BasicGraph::getCurLC(void)
{
	return cur_lc ;
}

NODEID BasicGraph::lastNodeLC(LAYOUTCOMPID lc_id)
{
	BasicNode*	temp_node ;

	(*(lc_id->ch_nodelist)).getLastNode(temp_node) ;
	return temp_node->id ;
}

LAYOUTCOMPID BasicGraph::getRootLC(void)
{
	LAYOUTCOMPID	temp_lc ;
	if ((*lc_list).getFirstNode(temp_lc))
	    return temp_lc ;
	else {
	    _sw_errno = SWE_INTN ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(_sw_errno, "Swan:%s\n", _errorstr) ;
		return NULL;//NOTE:  VC++ requires that all control paths return a value
			       //       I arbitrarily choose null for the moment  *****
	}
}

// findCCA -- find the Closest Common Ancester
// of two layout components lc1 and lc2 in the same graph

LAYOUTCOMPID BasicGraph::findCCA(LAYOUTCOMPID lc1, LAYOUTCOMPID lc2)
{
	Stack<BasicLayoutComp*> s1, s2 ;
	// LCSTACK		s1, s2 ;
	LAYOUTCOMPID	root = getRootLC() ;
	LAYOUTCOMPID	temp_lc1, temp_lc2, cca_lc ;

	temp_lc1 = lc1 ;
	while (temp_lc1 != root) {
	    s1.push(temp_lc1) ;
	    temp_lc1 = (*temp_lc1).getPID() ;
	}

	temp_lc2 = lc2 ;
	while (temp_lc2 != root) {
	    s2.push(temp_lc2) ;
	    temp_lc2 = (*temp_lc2).getPID() ;
	}

	cca_lc = root ;
	while (!s1.isempty() && !s2.isempty()) {
	    s1.pop(temp_lc1) ;
	    s2.pop(temp_lc2) ;
	    if (temp_lc1 != temp_lc2)
		break ;
	    cca_lc = temp_lc1 ;
	}
	return cca_lc ;
}
	
/***********************************************************
 * End of graph.c 					   *
 ***********************************************************/
