/***********************************************************
 * node.c --- Member functions of class BasicNode          *
 * Jun Yang, November 17, 1994	  			   *
 ***********************************************************/

/*--------------- Header Files ---------------*/

#include <iostream.h>
#include <stdio.h>
#include <string.h>
extern "C" {
#include "geosim.h"
#include "gspriv.h"
}
#include "swan.h"

/*-------------- Member Functions of BasicNode ---------------*/

// constructors and destructor

BasicNode::BasicNode(NODETYPE node_type, NODEID node_id, LABEL name)
{
	register int	i ;

	id = node_id ;
	type = node_type ;
	lc_id = NULL ;
        rpos = ppos = wpos = UnsetPos ;
	if (name != NULL) {
	    label = new char[strlen(name)+1] ;
	    strcpy(label, name) ;
	}
	else 
	    label = NULL ;
	active = FALSE ;
	traversed = FALSE ;
	info = new NODEINFO ;
	nblist = new NODENBLIST ;
	
	for (i = 0 ; i < MAXBUSLINE ; i++) {
	    rbus[i] = FALSE ;
	    bbus[i] = FALSE ;
	}
}

BasicNode::BasicNode(void)
{
	register int	i ;

	lc_id = NULL ;
        rpos = UnsetPos ;
        ppos = UnsetPos ;
	label = NULL ;
	active = FALSE ;
	info = new NODEINFO ;
	nblist = new NODENBLIST ;
	
	for (i = 0 ; i < MAXBUSLINE ; i++) {
	    rbus[i] = FALSE ;
	    bbus[i] = FALSE ;
	}
}

BasicNode::~BasicNode(void)
{
	struct _nb_node *temp_nb ;
	bool		ok ;

	if (label != NULL) {
	    delete[] label ;
	    label = NULL ;
	}
	delete info ;
	info = NULL ;
	
	if (nblist != NULL) {
	    ok = (*nblist).getFirstNode(temp_nb) ;
	    while (ok) {
		temp_nb->id = NULL ;
		temp_nb->edge_ptr = NULL ;
		delete temp_nb ;
	        ok = (*nblist).getNextNode(temp_nb) ;
	    }
	    delete nblist ; 
	    nblist = NULL ;
	}
}

// member access functions

NODEID BasicNode::getID(void)
{
	return id ;
}

bool BasicNode::setLayoutComp(LAYOUTCOMPID layout_id)
{
	lc_id = layout_id ;
	return TRUE ;
}

LAYOUTCOMPID BasicNode::getLayoutComp(void)
{
	return lc_id ;
}

bool BasicNode::setAttr(NODEINFO* attrs)
{
	*info = *attrs ;
	return TRUE ;
}

NODEINFO* BasicNode::getAttr(void)
{
	return info ;
}

bool BasicNode::setRPos(LOCATION pos)
{
	rpos = pos ;
	return TRUE ;
}

LOCATION BasicNode::getRPos(void)
{
	return rpos ;
}

bool BasicNode::setPPos(LOCATION pos)
{
	ppos = pos ;
	return TRUE ;
}

LOCATION BasicNode::getPPos(void)
{
	return ppos ;
}

bool BasicNode::setColor(COLORTYPE color) 
{
	info->color = color ;
	return TRUE ;
}

COLORTYPE BasicNode::getColor(void) 
{
	return info->color ;
}

bool BasicNode::setBoxSize(BOXSIZE size)
{
	info->size = size ;
	return TRUE ;
}

BOXSIZE BasicNode::getBoxSize(void)
{
	return info->size ;
}

bool BasicNode::setFilled(FLAG f_flag)
{
	info->filled = f_flag ;
	return TRUE ;
}

FLAG BasicNode::getFilled(void)
{
	return info->filled ;
}

bool BasicNode::setType(NODETYPE n_type)
{
	type = n_type ;
	return TRUE ;
}

NODETYPE BasicNode::getType(void)
{
	return type ;
}

bool BasicNode::setWidth(int n_width)
{
	info->size.w = n_width ;
	return TRUE ;
}

int BasicNode::getWidth(void)
{
	return info->size.w ;
}

bool BasicNode::setHeight(int n_height)
{
	info->size.h = n_height;
	return TRUE ;
}

int BasicNode::getHeight(void)
{
	return info->size.h ;
}

bool BasicNode::setLineTH(int line_TH)
{
	info->lineTH = line_TH ;
	return TRUE ;
}

int BasicNode::getLineTH(void)
{
	return info->lineTH ;
}

FLAG BasicNode::isDisplayed(void)
{
	return active ;
}

void BasicNode::enActive(void)
{
	active = TRUE ;
}

void BasicNode::disActive(void)
{
	active = FALSE ;
}

int BasicNode::getRBus(void)
{
	register int i;
	
	for (i = 1 ; i < MAXBUSLINE ; i++) 
	    if (rbus[i] == FALSE) {
		rbus[i] = TRUE ;
		return i ;
	    }
	return -1 ;
}

int BasicNode::getBBus(void)
{
	register int i;
	
	for (i = 1 ; i < MAXBUSLINE ; i++) 
	    if (bbus[i] == FALSE) {
		bbus[i] = TRUE ;
		return i ;
	    }
	return -1 ;
}

bool BasicNode::setLabel(LABEL new_label)
{
	if (new_label != NULL) {
	    if (label != NULL) 
		delete[] label ;
	    label = new char[strlen(new_label)+1] ;
	    strcpy(label, new_label) ;
	}
	else {
	    if (label != NULL) 
		delete[] label ;
	    label = NULL ;
	}
	return TRUE ;
}

LABEL BasicNode::getLabel(void) 
{
	return label ;
}

void BasicNode::setTravFlag(void)
{
	traversed = TRUE ;
}

void BasicNode::clearTravFlag(void)
{
        traversed = FALSE ;
}

bool BasicNode::isTraversed(void)
{
	return traversed ;
}

// Manipulations of Nodes/Edges 
//	. getEdge
//	. getNBLis

BasicEdge* BasicNode::getEdge(BasicNode* nb_node)
{
	struct _nb_node  *temp_nb ;
	bool		 ok ;

	ok = (*nblist).getFirstNode(temp_nb) ;
	while (ok) {
	    if (temp_nb->id == nb_node) 
		return temp_nb->edge_ptr ;
	    ok = (*nblist).getNextNode(temp_nb) ;
	}
	return NULL ;
}

NODENBLIST* BasicNode::getNBList(void)
{
	return nblist ;
}

// pick function

bool BasicNode::inThisNode(int x, int y)
{
	if (!isDisplayed())
	    return FALSE ;

	if (x < wpos.x || x > wpos.x + wsize.w)
	    return FALSE ;
	if (y < wpos.y || y > wpos.y + wsize.h)
	    return FALSE ;
	_sw_pick_node = this ;
        _sw_pick_graph = (*(*_sw_pick_node).getLayoutComp()).getGID() ;
	return TRUE ;
}
	
// Input/Output 
//	. save		
//	. load

bool BasicNode::save(ofstream& node_out)
{
	if (node_out == NULL)
	    return FALSE ;

	node_out << '(' << (int)id << ' ' << (*lc_id).getEID() 
		 << ' ' << type << ' ' << rpos.x << ' ' 
		 << rpos.y << ' ' << '\"' ;
	if (label != NULL)
	    node_out << label ;
 	node_out << '\"' << ' ' 
		 << info->size.w << ' ' << info->size.h << ' '
		 << ' ' << info->color << ' '
		 << info->filled << ' ' << info->lineTH 
		 << ')' << '\n' ;
	return TRUE ;
}

bool BasicNode::load(ifstream& node_in, BasicLayoutComp *parent)
{
	char	c, *p ;
	char	temp_str[80] ;
	LC_ID	temp_lcid ;
	int	temp_id, temp_len ;

	if (node_in == NULL)
	    return FALSE ;
	node_in >> c ;
	if (c != '(')
	    return FALSE ;
	node_in >> temp_id ;
	id = (NODEID)temp_id ;
	node_in >> temp_lcid ;
	if (temp_lcid != (*parent).getEID()) 
	    return FALSE ;
	lc_id = parent ;
	node_in >> type >> rpos.x >> rpos.y >> c ;
	if (c != '\"')
	    return FALSE ;
	p = temp_str ;
	node_in >> c ;
	while (c != '\"') {
	    *p++ = c ;
	    node_in >> c ;
	}
	*p = '\0' ;
	temp_len = strlen(temp_str) ;
	if (label == NULL) 
	    label = new char[temp_len+1] ;
	else if (strlen(label) < temp_len) {
	    delete[] label ;
	    label = new char[temp_len+1] ;
	}
	strcpy(label, temp_str) ;

	node_in >> info->size.w >> info->size.h 
		>> info->color >> info->filled >> info->lineTH >> c ;
	if (c != ')')
	    return FALSE ;
	return TRUE ;
}

// Display member functions
//	. displayNode	
//	. pan	
//	. zoom	

bool BasicNode::displayNode()
{
	LOCATION	temp_pos, temp_pos2 ;

	if (!active) {
	    temp_pos = getRPos() ;
	    temp_pos2 = (*lc_id).getPPos() ;
	    ppos.x = temp_pos.x + temp_pos2.x ;
	    ppos.y = temp_pos.y + temp_pos2.y ;
	    wpos.x = (int)(ppos.x * zoom_factor) - sc_x_origin ;
	    wpos.y = (int)(ppos.y * zoom_factor) - sc_y_origin ;
	    wsize.w = (int)(info->size.w * zoom_factor) ;
	    wsize.h = (int)(info->size.h * zoom_factor) ;
	    active = TRUE ;
	}
	return dump() ;
}

bool BasicNode::dump()
{
	int		temp_w, temp_h, temp_r ;
	LOCATION	temp_pos, temp_pos2 ;
	int		wsize12, wsize34, n ;
	BasicGraph	*temp_g ;

	if (!active)
	    return FALSE ;

	// CAS: This is supposed to fix a bug with drawing nodes within
	// the wrong window context.
	GSsetcurrentnamedwindow(SWG_SHOWWIN);

	swg_setcurrentcolor(info->color) ;
	swg_setcurrentsize(info->lineTH) ;

	GSsetfont(SMALL) ;

	switch (type) {
	case BOX:
	    if (info->filled) 
	        swg_drawrectangle(wpos.x, wpos.y, wsize.w, wsize.h,
                                                      FILLEDRECT) ;
	    else              
	        swg_drawrectangle(wpos.x, wpos.y, wsize.w, wsize.h,
                                                      FRAMEDRECT) ;
	    if (zoom_factor > ZOOMOUTSTR) {
	        swg_setcurrentcolor(BLACK) ;
	        swg_drawstring(wpos.x + wsize.w/2 - strlen(label)*3,
                               wpos.y + wsize.h/2 + 4, label) ;
	    }
            break ;

	case TABOXL:
	    wsize12 = wsize.w / 2 ;
	    wsize34 = wsize.w * 3 / 4 ;
            swg_drawrectangle(wpos.x, wpos.y, wsize.w, wsize.h,
                                                      FRAMEDRECT) ;
	    swg_drawline(wpos.x + wsize12, wpos.y, 
			 wpos.x + wsize12, wpos.y + wsize.h - 1) ;
	    swg_drawline(wpos.x + wsize34, wpos.y, 
			 wpos.x + wsize34, wpos.y + wsize.h - 1) ;
	    n = (*nblist).getNodeCount() ;
	    switch (n) {
	    case 0: // last node in a directed graph
		swg_drawline(wpos.x + wsize12, wpos.y + wsize.h,
			     wpos.x + wsize34, wpos.y) ;
		swg_drawline(wpos.x + wsize34, wpos.y + wsize.h,
			     wpos.x + wsize.w, wpos.y) ;
		break ;

	    case 1: 
		temp_g = (*lc_id).getGID() ;
		if ((*temp_g).isDirected()) {
		    if ((*lc_id).searchNBLC(this) != NULL) 
	            switch ((*lc_id).getType()) {
		    case LISTDOWN:
	                swg_drawline(wpos.x + wsize12, 
				     wpos.y + wsize.h,
                                     wpos.x + wsize34, wpos.y) ;
			break ;
		    case LISTACROSS:
			swg_drawline(wpos.x + wsize34, wpos.y + wsize.h,
                            	     wpos.x + wsize.w, wpos.y) ;
			break ;
		    default:
			break ;
		    }
		    else 
	            switch ((*lc_id).getType()) {
		    case LISTDOWN:
	                swg_drawline(wpos.x + wsize34, 
				     wpos.y + wsize.h,
                                     wpos.x + wsize.w, wpos.y) ;
			break ;
		    case LISTACROSS:
			swg_drawline(wpos.x + wsize12, wpos.y + wsize.h,
                            	     wpos.x + wsize34, wpos.y) ;
			break ;
		    default:
			break ;
		    }
		}
		else { // undirected -- not implemented
		    ;
		}
		break ;

	    default:
		break ;
	    }
	    if (zoom_factor > ZOOMOUTSTR) {
	        swg_setcurrentcolor(BLACK) ;
	        swg_drawstring(wpos.x + XOFFSETTEXT,
                               wpos.y + wsize.h/2 + 4, label) ;
	    }
            break ;

	case CIRCLE:
	    if (info->filled)
                swg_drawellipse(wpos.x, wpos.y, 
                       	        wsize.w, wsize.h, FILLEDELLIPSE) ;
	    else                
                swg_drawellipse(wpos.x, wpos.y, 
                       	        wsize.w, wsize.h, FRAMEDELLIPSE) ;
	    if (zoom_factor > ZOOMOUTSTR) {
                swg_setcurrentcolor(BLACK) ;
                swg_drawstring((int)(wpos.x+wsize.w/2-strlen(label)*3),
                   int(wpos.y+4+wsize.h/2), label) ;
	    }
            break ;

	default:
	    break ;

	}
	return TRUE ;
}

bool BasicNode::pan(int xoffset, int yoffset)
{
	wpos.x += xoffset ;
	wpos.y += yoffset ;
	return TRUE ;
}

bool BasicNode::zoom(void)
{
	wpos.x = (int)(ppos.x * zoom_factor) - sc_x_origin ;
	wpos.y = (int)(ppos.y * zoom_factor) - sc_y_origin ;
	wsize.w = (int)(info->size.w * zoom_factor) ;
	wsize.h = (int)(info->size.h * zoom_factor) ;
	return TRUE ;
}

/***********************************************************
 * End of node.c 					   *
 ***********************************************************/
