/************************************************************
 * swgintr.c -- Swan Graphical Display Library Functions    *
 * Jun Yang, November 17, 1994				    *
 ************************************************************/

/*--------------- Header Files ---------------*/

#include <iostream.h>
#include <stdio.h>
extern "C" {
#include "geosim.h"
}
#include "swan.h"
#include "laykk.h"


/*-------------- Global Variables ---------------*/

LINETYPE    swg_currentLineType ;
RECTTYPE    swg_currentRectType ;
ELLIPSETYPE swg_currentEllipseType ;
ARROWTYPE   swg_currentArrowType ;
LOCATION    swg_currentPos ;
COLORTYPE   swg_currentColor ;
int 	    swg_currentsize ;
FONTTYPE    swg_currentFont ;

// SWG Color tables
char* color_table[]={
        "black",
        "dgray",
        "mgray",
        "lgray",
        "bluegray",
        "lblue",
        "peach",
        "lcyan",
        "mcyan",
        "gray",
        "myellow",
        "lyellow",
        "magenta",
        "dgreen",
        "pasgreen"
} ;

// SWG font table
FONTTYPE font_table[]={
	{ "small", { 7, 10 }}
} ;

/*--------------- Swan graphics library functions ---------------*/

bool swg_drawline(LOCATION pos1, LOCATION pos2) 
{
        GSwdrawline(pos1.x, pos1.y, pos2.x, pos2.y) ;
	return TRUE ;
}

bool swg_drawline(int x1, int y1, int x2, int y2) 
{
        GSwdrawline(x1, y1, x2, y2) ;                    
	return TRUE ;
}

bool swg_drawcurve(CURVE* path)
{
	LOCATION	pos1 ;
	LOCATION	pos2 ;
	bool		ok ;

	ok = (*path).getFirstNode(pos1) ;
	if (ok) {
	    ok = (*path).getNextNode(pos2) ;
	    while (ok) {
		swg_drawline(pos1, pos2) ;
		pos1 = pos2 ;
	        ok = (*path).getNextNode(pos2) ;
	    }
	}
	return TRUE ;
}

bool swg_drawrectangle(int x, int y, int width, int height, 
			  RECTTYPE recttype) 
{
        switch (recttype) {
        case FRAMEDRECT:
            GSwdrawrect(x, y, width, height) ; 
            break ;

        case FILLEDRECT:
            GSwfillrect(x, y, width, height) ;
            break ;

        default:
            _sw_errno = SWE_INVRECT ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
            return FALSE ;
        }
	return TRUE ;
}

bool swg_drawellipse(int x, int y, int width, int height, 
			ELLIPSETYPE ellipsetype) 
{
        switch (ellipsetype) {
        case FRAMEDELLIPSE:
            GSwdrawellipse(x, y, width, height) ;
            break ;

        case FILLEDELLIPSE:
            GSwfillellipse(x, y, width, height) ;
            break ;

        default:
            _sw_errno = SWE_INVELLIPSE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
            return FALSE ;
        }
	return TRUE ;
}

bool swg_drawstring(int x, int y, char *str)
{
	if (str != NULL) 
            GSwwritemsg(x, y, str) ;
	return TRUE ;
}

bool swg_setcurrentlinetype(LINETYPE linetype) 
{
	swg_currentLineType = linetype ;	
	return TRUE ;
}

bool swg_setcurrentrecttype(RECTTYPE recttype) 
{
	swg_currentRectType = recttype ;
	return TRUE ;
}

bool swg_setcurrentellipse(ELLIPSETYPE ellipsetype) 
{
	swg_currentEllipseType = ellipsetype ;
	return TRUE ;
}

bool swg_setcurrentpos(LOCATION pos) 
{
	swg_currentPos = pos ;
	return TRUE ;
}

bool swg_setcurrentarrowtype(ARROWTYPE arrowtype) 
{
	swg_currentArrowType = arrowtype ;
	return TRUE ;
}

bool swg_setcurrentcolor(COLORTYPE newcolor) 
{
	GSColor		lcolor ;

	swg_currentColor = newcolor ;
	lcolor = GSconvertcolor(color_table[swg_currentColor]) ;	
	GSsetcolor(lcolor) ;
	return TRUE ;
}

bool swg_setcurrentsize(int thick) 
{
	swg_currentsize = thick ;
	GSsetlinesize(swg_currentsize) ;
	return TRUE ;
}

bool swg_setcurrentfont(FONTTYPE fonttype) 
{
	swg_currentFont = fonttype ;
	return TRUE ;
}

bool swg_refresh()
{
	bool		ok ;
	BasicGraph*	g ;

        GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
        GSredrawnamedwindow(SWG_SHOWWIN) ;
        GSdoeventsandreturn() ;

	if (ggl != NULL) {
	    ok = (*ggl).getFirstNode(g) ;
	    while (ok) {
	        (*g).dump() ;
	        ok = (*ggl).getNextNode(g) ;
	    }
	}
	return TRUE ;
}

bool swg_pan(int xoffset, int yoffset)
{
	bool		ok ;
	BasicGraph*	g ;

	ok = (*ggl).getFirstNode(g) ;
	while (ok) {
	    (*g).pan(xoffset, yoffset) ;
	    ok = (*ggl).getNextNode(g) ;
	}
	return TRUE ;
}
	
bool swg_zoom(void)
{
	bool		ok ;
	BasicGraph*	g ;

	if (ggl != NULL) {
	    ok = (*ggl).getFirstNode(g) ;
	    while (ok) {
	        (*g).zoom() ;
	        ok = (*ggl).getNextNode(g) ;
	    }
	}
	return TRUE ;
}

bool swg_clear(LOCATION pos, BOXSIZE bsize)
{
	GSColor		lcolor ;
	
	GSsetcurrentnamedwindow("SWG_SHOWWIN") ;
	lcolor = GSconvertcolor("white") ;
	GSsetcolor(lcolor) ;
	GSwfillrect(pos.x, pos.y, bsize.w, bsize.h) ;
	GSdoeventsandreturn() ;
	return TRUE ;
}

bool swg_clearpath(CURVE* path)
{
	GSColor		lcolor ;
	
	GSsetcurrentnamedwindow("SWG_SHOWWIN") ;
	lcolor = GSconvertcolor("white") ;
	GSsetcolor(lcolor) ;
	swg_drawcurve(path) ;
	return TRUE ;
}

// pick an element from the graph

bool swg_pick(int x, int y, PICKTYPE ptype) 	
{
        int     	text_x, text_y ;
        GSColor 	color ;
	char		temp_str[80] ;
	LCLIST*		temp_lclist ;
	bool		ok ;
	BasicGraph	*temp_g ;
	BasicLayoutComp *temp_lc ;
        int dumx, dumy;

	if (_sw_root_lc == NULL)
	    return FALSE ;

	switch (ptype) {
	case PICKNODE:
	  if ((*_sw_root_lc).inThisLC(x, y, ptype)) { 
	    if (fWaitNodePicked || fWaitNodeChangeAttr)
		return TRUE ;
            dumx = x+5; dumy = y+31;
            GSsetwindowspecs("nodeinfo", &dumx, &dumy, NULL, NULL) ;
            GSdrawnamedwindow("nodeinfo", BSTORE, POPUP) ;
            GSgetfieldrect("nodeid", &text_x, &text_y, NULL, NULL,
                            &color) ;
            GSsetcolor(color) ;
	    sprintf(temp_str, "ID = %ld", (long)(*_sw_pick_node).getID()) ;
            GSwwritemsg(text_x+10, text_y+10, temp_str) ;
            GSgetfieldrect("nodestr", &text_x, &text_y, NULL, NULL,
                            &color) ;
            GSsetcolor(color) ;
	    sprintf(temp_str, "INFO: %s", (*_sw_pick_node).getLabel()) ;
            GSwwritemsg(text_x+10, text_y+10, temp_str) ;
	    GSdoeventsandreturn() ;
	    return TRUE ;
	  }
	  return FALSE ;

	case PICKEDGE:
	  ok = (*ggl).getFirstNode(temp_g) ;
	  while (ok) {
	  if ((*temp_g).edgeIn(x, y)) { 
	    if (fWaitEdgePicked || fWaitEdgeChangeAttr)
		return TRUE ;
            dumx = x+5; dumy = y+31;
            GSsetwindowspecs("edgeinfo", &dumx, &dumy, NULL, NULL) ;
            GSdrawnamedwindow("edgeinfo", BSTORE, POPUP) ;
            GSgetfieldrect("id1", &text_x, &text_y, NULL, NULL,
                            &color) ;
            GSsetcolor(color) ;
	    sprintf(temp_str, "ID1 = %ld", 
	        (long)(*((*_sw_pick_edge).getSNode())).getID()) ;
            GSwwritemsg(text_x+10, text_y+10, temp_str) ;
            GSgetfieldrect("id2", &text_x, &text_y, NULL, NULL,
                            &color) ;
            GSsetcolor(color) ;
	    sprintf(temp_str, "ID2 = %ld", 
	        (long)(*((*_sw_pick_edge).getENode())).getID()) ;
            GSwwritemsg(text_x+10, text_y+10, temp_str) ;
            GSgetfieldrect("edgestr", &text_x, &text_y, NULL, NULL,
                            &color) ;
            GSsetcolor(color) ;
	    sprintf(temp_str, "INFO: %s", (*_sw_pick_edge).getLabel()) ;
            GSwwritemsg(text_x+10, text_y+10, temp_str) ;
	    GSdoeventsandreturn() ;
	    return TRUE ;
	  }
	  ok = (*ggl).getNextNode(temp_g) ;
	  }
	  return FALSE ;

	case PICKGRAPH:
	  ok = (*ggl).getFirstNode(temp_g) ;
	  while (ok) {
	      temp_lc = (*temp_g).getRootLC() ;
	      if ((*temp_lc).inThisLC(x, y, PICKGRAPH))
		  return TRUE ;
	      ok = (*ggl).getNextNode(temp_g) ;
	  }
	  return FALSE ;

	default:
	  return FALSE ;
	}
}

bool swg_swapnode(BasicNode* node1, BasicNode* node2)
{
	LOCATION 	temp_pos ;
	LAYOUTCOMPID	temp_lc = (*node1).getLayoutComp() ;

	if (temp_lc != (*node2).getLayoutComp())
	    return FALSE ;

	switch ((*(*temp_lc).getGID()).getLayoutType()) {
	case CIRCLENET:
	case KKNET:
	    break ;
	default:
	    return FALSE ;
	}
	temp_pos = (*node1).getRPos() ;
	(*node1).setRPos((*node2).getRPos()) ;
	(*node2).setRPos(temp_pos) ;
	return TRUE ;
}
	
bool swg_movenode(BasicNode* node1, int pos_x, int pos_y)
{
	LOCATION 	temp_pos, temp_pos2 ;
	BasicLayoutComp	*temp_lc ;
	BasicGraph	*temp_graph ;

	if ((temp_lc = (*node1).getLayoutComp()) == NULL)
	    return FALSE ;

	temp_graph = (*temp_lc).getGID() ;
	switch ((*temp_graph).getLayoutType()) {
	case KKNET:
	    break ;
	default:
	    return FALSE ;
	}

	temp_pos = (*temp_lc).getPPos() ;
	temp_pos2.x = (int) ((pos_x+sc_x_origin)/zoom_factor) -
			    temp_pos.x ;
	temp_pos2.y = (int) ((pos_y+sc_y_origin)/zoom_factor) -
			    temp_pos.y ;
	(*node1).setRPos(temp_pos2) ;
	(*node1).disActive() ;

	_sw_normalizeRPos(temp_graph) ;
	switch ((*temp_graph).getLayoutType()) {
	case KKNET:
	    _KK_calboxsize((*temp_lc).getGID()) ; 
	    break ;
	default:
	    break ;
	}
	return TRUE ;
}

bool swg_savegraph(char* graph_name, BasicGraph* g_ptr)
{
	ofstream gout(graph_name) ;
	if (!gout)
	    return FALSE ;
	if (g_ptr == NULL)
	    return FALSE ;
	if (!(*g_ptr).save(gout))
	    return FALSE ;
	return TRUE ; 
} 

bool swg_loadgraph(char* graph_name)
{
	BasicGraph	*g_ptr ;	
	ifstream	gin(graph_name) ;

	if (!gin) 
	    return FALSE ;

	g_ptr = new BasicGraph(NULLGRAPHID) ;
	if (g_ptr == NULL)
	    return FALSE ;
	if (!(*g_ptr).load(gin))
	    return FALSE ;
	(*ggl).insert(g_ptr) ;
	return TRUE ; 
} 

/************************************************************
 * End of swgintr.c					    *
 ************************************************************/
