/************************************************************
 * swintr.c -- Swan Annotation Interface Library	    *
 * Jun Yang, November 17, 1994				    *
 ************************************************************/

/*--------------- Header Files -----------*/
#ifndef UNIX
#include <windows.h>
#endif
#include <iostream.h>
#include <string.h>
#include <math.h>
#include <new.h>
extern "C" {		
#include "geosim.h"
}
#include "swan.h"
#include <stdarg.h>

/*--------------- Global Variables ---------------*/

BasicGraph*	_sw_pick_graph ; 
BasicNode*	_sw_pick_node ;
BasicEdge*	_sw_pick_edge ;
LOCATION	_sw_pick_pos ;
char		_sw_pick_string[80] ;

char		*_sw_menu_list[] = {
	"edit_menu"
} ;

/*---------- Swan Annotation Interface Library Functions ----------*/

bool sw_init(GS_MAIN_ARGS_F)
{
	// handle out of memory exception 
/*	set_new_handler(&out_of_store) ; */

        GSpermit_log = FALSE; // Turn off GIL's logging

	if (!fSwanWinInited) {	// Initialize Swan window
	    _sw_wininit(GS_MAIN_ARGS_A) ; 
	    fSwanWinInited = TRUE ;
	}    
	if (!fSwanInited) { // Initialize Swan internal structures
	    _sw_init() ;
	    fSwanInited = TRUE ;
	}

	return TRUE ;
}

void sw_quit(void)
{
	sw_clear() ;	// clear graphs created in Swan
	if (fSwanWinInited) { // close Swan windows
	    GSquit(NULL) ;
	    fSwanWinInited = FALSE ;
	}
}
	
void sw_clear(void)
{
	/* suspicious -- the function name may need to be changed */
	if (fSwanInited) { 
	    _sw_quit() ;	// delete all graphs created in Swan
	    fSwanInited = FALSE ;
	}
}
	
int sw_wait(int wButton)
{
	if (wButton & RUN)
	    fWaitRun = TRUE ;
	if (wButton & STEP)
	    fWaitStep = TRUE ;
	if (wButton & INSNODE)
	    fWaitInsNode = TRUE ;
	if (wButton & DELNODE)
	    fWaitDelNode = TRUE ;
	if (wButton & INSEDGE)
	    fWaitInsEdge = TRUE ;
	if (wButton & DELEDGE)
	    fWaitDelEdge = TRUE ;
	while (TRUE) {
	    GSdoeventsandreturn() ;
	    if ((wButton & RUN) && (!fWaitRun))
		return RUN ;
	    if ((wButton & STEP) && (!fWaitStep))
		return STEP ;
	    if ((wButton & INSNODE) && (!fWaitInsNode))
		return INSNODE ;
	    if ((wButton & DELNODE) && (!fWaitDelNode))
		return DELNODE ;
	    if ((wButton & INSEDGE) && (!fWaitInsEdge))
		return INSEDGE ;
	    if ((wButton & DELEDGE) && (!fWaitDelEdge))
		return DELEDGE ;
	}
}

void sw_enablebuttons(int wButton)
{
	if (wButton & RUN)
	    GSenablebutton("root", "Run", DRAW) ;
	if (wButton & STEP)
	    GSenablebutton("root", "Step", DRAW) ;
	GSdoeventsandreturn() ;
}

void sw_disablebuttons(int wButton)
{
	if (wButton & RUN)
	    GSdisablebutton("root", "Run", DRAW) ;
	if (wButton & STEP)
	    GSdisablebutton("root", "Step", DRAW) ;
	GSdoeventsandreturn() ;
}

bool sw_enablemenuitem(int menu, int item)
{
    GSenablemenuitem(SW_MENU[menu], SW_MENUITEM[item]) ;
    GSdoeventsandreturn() ;
    return TRUE ;
}
	    
bool sw_disablemenuitem(int menu, int item)
{
    GSdisablemenuitem(SW_MENU[menu], SW_MENUITEM[item]) ;
    GSdoeventsandreturn() ;
    return TRUE ;
}
	    
GRAPHID	sw_newgraph(GRAPHID g_id, NODETYPE node_type, FLAG dir_graph)
{
	BasicGraph* handle = NULL ;

	if (g_id != NULLGRAPHID) {
	    if (_sw_searchgraph(g_id) != NULL) { 
		// g_id already been used
		_sw_errno = SWE_BADGRAPHID ;
		_sw_geterrormsg(_sw_errno, _errorstr) ;
		GSelog(0, "Swan:%s\n", _errorstr) ;
	 	return NULLGRAPHID ;
	    }
	    handle = new BasicGraph(g_id, node_type) ;
	    (*ggl).insert(handle) ;
	    (*handle).setDirected(dir_graph) ;
	    return g_id ;
	}
	else {
	    handle = new BasicGraph(NULLGRAPHID, node_type) ;
	    (*ggl).insert(handle) ;
	    (*handle).setDirected(dir_graph) ;
	    return (*handle).getID() ;
	}
}

bool	sw_deletegraph(GRAPHID g_id)
{
	BasicGraph*	handle = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
            _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
            return FALSE ;
	}
	delete handle ;
	(*ggl).remove(handle) ;
	_sw_check_buttons() ;
	return TRUE ;
}
	    
LC_ID sw_newlc(GRAPHID g_id, LC_ID lc_id, LAYOUTCOMPTYPE lc_type)
{
	BasicGraph*	handle = NULL ;
	LC_ID		temp_lcid ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
            _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
            return NULLLCID ;
	}
	temp_lcid = (*handle).createLC(lc_id, lc_type) ;
	if (temp_lcid != NULLLCID) 
	    _sw_check_buttons() ;
	return temp_lcid ;
}

bool sw_deletelc(GRAPHID g_id, LC_ID lc_id)
{
	BasicGraph*	handle = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	if ((*handle).removeLC(lc_id)) {
	    _sw_check_buttons() ;
	    return TRUE ;
	} 
	return FALSE ;
}

bool sw_setcurlc(GRAPHID g_id, LC_ID lc_id)
{
	BasicGraph*	handle = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	return (*handle).setCurLC(lc_id) ;
}

// suspicious: this function may need to be taken out of SAIL

bool sw_displaygraph(GRAPHID g_id)
{
	BasicGraph*	handle = NULL ;
	BasicGraph*	temp_graph ;
	bool		ok ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

/*
	GSredrawnamedwindow(SWG_SHOWWIN) ;
	GSdoeventsandreturn() ;	

	if (!(*handle).isDisplayed() || (*handle).isNewLayout())
	    (*handle).layout() ;
	(*handle).setNewLayout(FALSE) ;
	_sw_set_allgraphs_pos() ;
	ok = (*ggl).getFirstNode(temp_graph) ;
	while (ok) {
	    if (temp_graph == handle) 
		(*temp_graph).displayGraph() ;
	    else {
		if ((*temp_graph).isDisplayed() &&
		    (*temp_graph).isMoved()) {
		    (*temp_graph).displayGraph() ;
		    (*temp_graph).setMovedFlag(FALSE) ;
		}
	    }
	(*handle).displayGraph() ;
	GSdoeventsandreturn() ;	
	return TRUE ;
*/
	sw_displayallgraphs() ;
	return TRUE ;
}
	
void sw_displayallgraphs(void)
{
	bool		ok ;
	BasicGraph*	graph_ptr ;

	GSredrawnamedwindow(SWG_SHOWWIN) ;
	GSdoeventsandreturn() ;	

	if (ggl != NULL) {
	    ok = (*ggl).getFirstNode(graph_ptr) ;
	    while (ok) {
	        if ((*graph_ptr).isNewLayout() || 
		    !(*graph_ptr).isDisplayed()) {
	            (*graph_ptr).layoutGraph() ;
		    (*graph_ptr).setNewLayoutFlag(FALSE) ;
		}
	    	ok = (*ggl).getNextNode(graph_ptr) ;
	    }
	    _sw_set_allgraphs_pos() ;
	    ok = (*ggl).getFirstNode(graph_ptr) ;
	    while (ok) {
	        (*graph_ptr).displayGraph() ;
	    	ok = (*ggl).getNextNode(graph_ptr) ;
	    }
	}
}

bool sw_insertnode(GRAPHID g_id, NODEID node_id, LABEL info)
{
	BasicGraph*	handle = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	if ((*handle).insertNode(node_id, info)) {
	    if ((*handle).isAutoRelayout())
	        (*handle).setNewLayoutFlag(TRUE) ;
	    if (_sws_autoredraw && (*handle).isDisplayed())
	         sw_displayallgraphs() ;
	    return TRUE ;
	}
	return FALSE ;
}

bool sw_insertnodeedge(GRAPHID g_id, NODEID s_node, LABEL info1,
		          NODEID e_node, LABEL info2, LABEL info3)
{
	BasicGraph*	handle = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	(*handle).insertNode(s_node, info1) ;
	(*handle).insertNode(e_node, info2) ;
	(*handle).insertEdge(s_node, e_node, info3) ;
	if ((*handle).isAutoRelayout())
	    (*handle).setNewLayoutFlag(TRUE) ;
	if (_sws_autoredraw && (*handle).isDisplayed()) 
	    sw_displayallgraphs() ;
	return TRUE ;
}

bool sw_deletenode (GRAPHID g_id, NODEID node_id)
{
	BasicGraph*	handle = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	if ((*handle).deleteNode(node_id)) {
	    if ((*handle).isAutoRelayout())
	        (*handle).setNewLayoutFlag(TRUE) ;
	    if (_sws_autoredraw && (*handle).isDisplayed())
	        sw_displayallgraphs() ;
	    return TRUE ;
	}
	return FALSE ;
}

bool sw_insertedge (GRAPHID g_id, NODEID node1, NODEID node2, 
		       LABEL info)
{
	BasicGraph*	handle = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	if ((*handle).insertEdge(node1, node2, info)) {
	    if ((*handle).isAutoRelayout())
	        (*handle).setNewLayoutFlag(TRUE) ;
	    if (_sws_autoredraw && (*handle).isDisplayed())
	        sw_displayallgraphs() ;
	    return TRUE ;
	}
	return FALSE ;
}
	
bool sw_insertbinedge (GRAPHID g_id, NODEID node1, NODEID node2, 
		       LABEL info, FLAG child)
{
	BasicGraph*	handle = NULL ;
	BasicNode*	temp_node ;
	struct _nb_node *temp_nb ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	if ((*handle).insertEdge(node1, node2, info)) {
	    if ((temp_node = (*handle).searchNode(node1)) != NULL) {
		if ((temp_nb = searchNBNode(temp_node,
						node2)) != NULL) 
		    temp_nb->fChild = child ;
	    }
	    if ((*handle).isAutoRelayout())
	        (*handle).setNewLayoutFlag(TRUE) ;
	    if (_sws_autoredraw && (*handle).isDisplayed())
	        sw_displayallgraphs() ;
	    return TRUE ;
	}
	return FALSE ;
}

bool sw_deleteedge (GRAPHID g_id, NODEID node1, NODEID node2)
{
	BasicGraph*	handle = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	if ((*handle).deleteEdge(node1, node2)) {
	    if ((*handle).isAutoRelayout())
	        (*handle).setNewLayoutFlag(TRUE) ;
	    if (_sws_autoredraw && (*handle).isDisplayed())
	        sw_displayallgraphs() ;
	    return TRUE ;
	}
	return FALSE ;
}

/*------------ Graphical Attributes Setting ------------*
 *		. sw_setcurrentattr			*
 *		. sw_setgraphattr			*
 *		. sw_setlcattr				*
 *		. sw_setnodeattr			*
 *		. sw_setedgeattr			*
 *------------------------------------------------------*/

bool sw_setcurrentattr(INDEX g_attr, IVALUE val)
{
	switch (g_attr) {
	case GDIRECTED:
	    _sw_cur_fDirected = val ;
	    break ;

	case GNODETYPE:
	    _sw_cur_nodeType = val ;
	    break ;

	case GNODECOLOR:
	    _sw_cur_nodeColor = val ;      
	    break ;

	case GNODEFILLED:
	    _sw_cur_nodeFilled = val ;      
	    break ;

	case GNODEWIDTH:
	    _sw_cur_nodeWidth = val ;      
	    break ;

	case GNODEHEIGHT:
	    _sw_cur_nodeHeight = val ;       
	    break ;

	case GNODELINETH:
	    _sw_cur_nodeLineTH = val ;      
	    break ;

	case GEDGELENGTH:
	    _sw_cur_edgeLength = val ;      
	    break ;

	case GEDGELINETH:
	    _sw_cur_edgeLineTH = val ;         
	    break ;
	
	case GEDGECOLOR:
	    _sw_cur_edgeColor = val ;       
	    break ;
	
	case GAUTOREDRAW:
	    _sws_autoredraw = val ;
	    break ;

	default:
	    _sw_errno = SWE_NOATTR ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	return TRUE ;
}

bool sw_setgraphattr(GRAPHID g_id, INDEX g_attr, IVALUE val) 
{
	BasicGraph*	handle = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

	switch (g_attr) {
	case GDIRECTED:
	    (*handle).setDirected(val) ;
	    break ;

	case GNEWLAYOUT:
	    (*handle).setNewLayoutFlag(val) ;
	    break ;

	case GAUTORELAYOUT:
	    (*handle).setAutoRelayout(val) ;
	    break ;
	
	case GLAYOUTMODE:
	    (*handle).setLayoutMode(val) ;
	    break ;

	case GNODETYPE:
	    (*handle).setNodeType(val) ;
	    break ;

	case GNODECOLOR:
	    (*handle).setNodeColor(val) ;
	    break ;

	case GNODEFILLED:
	    (*handle).setNodeFilled(val) ;
	    break ;

	case GNODEWIDTH:
	    (*handle).setNodeWidth(val) ;
	    break ;

	case GNODEHEIGHT:
	    (*handle).setNodeHeight(val) ;
	    break ;

	case GNODELINETH:
	    (*handle).setNodeLineTH(val) ;
	    break ;

	case GEDGELENGTH:
	    (*handle).setEdgeLength(val) ;
	    break ;

	case GEDGELINETH:
	    (*handle).setEdgeLineTH(val) ;
	    break ;
	
	case GEDGECOLOR:
	    (*handle).setEdgeColor(val) ;
	    break ;
	
	case GEDGELABEL:
	    ((*handle).getGraphInfo())->fShowLabel = val ;
	    break ;

	default:
	    _sw_errno = SWE_NOATTR ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	// suspicious: some differences have to be made here
	if ((*handle).isDisplayed()) {
            (*handle).dump() ;
        }
        return TRUE ;
}

bool sw_getgraphattr(GRAPHID g_id, INDEX g_attr, IVALUE *val) 
{
	BasicGraph*	handle = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

	switch (g_attr) {
	case GDIRECTED:
	    *val = (*handle).isDirected() ;
	    break ;

	case GNEWLAYOUT:
	    *val = (*handle).isNewLayout() ;
	    break ;

	case GAUTORELAYOUT:
	    *val = (*handle).isAutoRelayout() ;
	    break ;
	
	case GLAYOUTMODE:
	    *val = (*handle).getLayoutMode() ;
	    break ;

	case GNODETYPE:
	    *val = (*handle).getNodeType() ;
	    break ;

	case GNODECOLOR:
	    *val = (*handle).getNodeColor() ;
	    break ;

	case GNODEFILLED:
	    *val = (*handle).getNodeFilled() ;
	    break ;

	case GNODEWIDTH:
	    *val = (*handle).getNodeWidth() ;
	    break ;

	case GNODEHEIGHT:
	    *val = (*handle).getNodeHeight() ;
	    break ;

	case GNODELINETH:
	    *val = (*handle).getNodeLineTH() ;
	    break ;

	case GEDGELENGTH:
	    *val = (*handle).getEdgeLength() ;
	    break ;

	case GEDGELINETH:
	    *val = (*handle).getEdgeLineTH() ;
	    break ;
	
	case GEDGECOLOR:
	    *val = (*handle).getEdgeColor() ;
	    break ;
	
	case GEDGELABEL:
	    *val = handle->getGraphInfo()->fShowLabel ;
	    break ;

	default:
	    _sw_errno = SWE_NOATTR ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
        return TRUE ;
}

bool sw_setlcattr(GRAPHID g_id, LC_ID lc_id, INDEX lc_attr, IVALUE val) 
{
	BasicGraph*	handle = NULL ;
	BasicLayoutComp* lc_ptr = NULL ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	if ((lc_ptr = (*handle).searchLC(lc_id)) == NULL) {
	    _sw_errno = SWE_NOLC ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

	switch (lc_attr) {
	case GNODETYPE:
	    (*lc_ptr).setNodeType(val) ;
	    break ;

	case GNODECOLOR:
	    (*lc_ptr).setNodeColor(val) ;
	    break ;

	case GNODEFILLED:
	    (*lc_ptr).setNodeFilled(val) ;
	    break ;

	case GNODEWIDTH:
	    (*lc_ptr).setNodeWidth(val) ;
	    break ;

	case GNODEHEIGHT:
	    (*lc_ptr).setNodeHeight(val) ;
	    break ;

	case GNODELINETH:
	    (*lc_ptr).setNodeLineTH(val) ;
	    break ;

	case GEDGELENGTH:
	    (*lc_ptr).setEdgeLength(val) ;
	    break ;

	case GEDGELINETH:
	    (*lc_ptr).setEdgeLineTH(val) ;
	    break ;
	
	case GEDGECOLOR:
	    (*lc_ptr).setEdgeColor(val) ;
	    break ;
	
	default:
	    _sw_errno = SWE_NOATTR ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	return TRUE ;
}

bool sw_setnodeattr(GRAPHID g_id, NODEID s_node, INDEX g_attr, ...)
{
	int		val ;
	LOCATION	pos, lc_pos ;
	LABEL		temp_str ;
	BasicNode*	node1 ;
	BasicGraph*	handle = NULL ;
	va_list		ap ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

	if ((node1 = (*handle).searchNode(s_node)) == NULL) {
	    _sw_errno = SWE_NONODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

	va_start(ap, g_attr) ;

	switch (g_attr) {	
	case GNODETYPE:
	    val = va_arg(ap, int) ;
	    (*node1).setType(val) ;
	    break ;

	case GNODECOLOR:
	    val = va_arg(ap, int) ;
	    (*node1).setColor(val) ;
	    break ;

	case GNODEFILLED:
	    val = va_arg(ap, int) ;
	    (*node1).setFilled(val) ;
	    break ;

	case GNODEWIDTH:
	    val = va_arg(ap, int) ;
	    (*node1).setWidth(val) ;
	    break ;

	case GNODEHEIGHT:
	    val = va_arg(ap, int) ;
	    (*node1).setHeight(val) ;
	    break ;

	case GNODELINETH:
	    val = va_arg(ap, int) ;
	    (*node1).setLineTH(val) ;
	    break ;

	case GNODELABEL:
	    temp_str = va_arg(ap, LABEL) ;
	    (*node1).setLabel(temp_str) ;
	    break ;

	case GNODEPOS:
	    pos.x = va_arg(ap, int) ;
	    pos.y = va_arg(ap, int) ;
	    if ((node1->getLayoutComp())->getType() == MANUAL) {
	        (*node1).setRPos(pos) ;
	        return TRUE ;
	    }
	    return FALSE ;
	
	default:
	    _sw_errno = SWE_NOATTR ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	va_end(ap) ;
	if ((*node1).isDisplayed()) {
            (*node1).displayNode() ;
        }
        return TRUE ;
}

bool sw_setedgeattr(GRAPHID g_id, NODEID src_node, NODEID dest_node, 
		       INDEX g_attr, ...)
{
	int		val ;
	LABEL		temp_str ;
	BasicEdge*	edge1 = NULL ;
	BasicGraph*	handle = NULL ;
	va_list		ap ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	
	if ((edge1 = (*handle).searchEdge(src_node, dest_node)) == NULL) {
	    _sw_errno = SWE_NOEDGE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

	va_start(ap, g_attr) ;

	switch (g_attr) {	
	case GEDGELABEL:
	    temp_str = va_arg(ap, LABEL) ;
	    (*edge1).setLabel(temp_str) ;
	    break ;

	case GEDGELENGTH:
	    val = va_arg(ap, int) ;
	    (*edge1).setLength(val) ;
	    break ;

	case GEDGELINETH:
	    val = va_arg(ap, int) ;
	    (*edge1).setLineTH(val) ;
	    break ;
	
	case GEDGECOLOR:
	    val = va_arg(ap, int) ;
	    (*edge1).setColor(val) ;
	    break ;
	
	default:
	    _sw_errno = SWE_NOATTR ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	// suspicious, depends on design consideration
 	// whether to redraw the edge immediately 
	if ((*edge1).isDisplayed()) {
	    GSsetcurrentnamedwindow(SWG_SHOWWIN) ;
            (*edge1).displayEdge() ;
        }
        return TRUE ;
}

bool sw_getedgeattr(GRAPHID g_id, NODEID src_node, NODEID dest_node, 
		       INDEX g_attr, ...)
{
	int		*val ;
	LABEL		temp_str ;
	BasicEdge*	edge1 = NULL ;
	BasicGraph*	handle = NULL ;
	va_list		ap ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	
	if ((edge1 = (*handle).searchEdge(src_node, dest_node)) == NULL) {
	    _sw_errno = SWE_NOEDGE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

	va_start(ap, g_attr) ;

	switch (g_attr) {
	case GEDGELABEL:
	    temp_str = va_arg(ap, LABEL) ;
	    strcpy(temp_str, (*edge1).getLabel()) ;
	    break ;

	case GEDGELENGTH:
	    val = va_arg(ap, int*) ;
	    *val = (*edge1).getLength() ;
	    break ;

	case GEDGELINETH:
	    val = va_arg(ap, int*) ;
	    *val = (*edge1).getLineTH() ;
	    break ;
	
	case GEDGECOLOR:
	    val = va_arg(ap, int*) ;
	    *val = (*edge1).getColor() ;
	    break ;
	
	default:
	    _sw_errno = SWE_NOATTR ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
        return TRUE ;
}

bool sw_getnodeattr(GRAPHID g_id, NODEID node_id, INDEX g_attr, ...)
{
	int		*val ;
	LABEL		temp_str ;
	LOCATION	pos, lc_pos ;
	BasicNode*	node1 ;
	BasicGraph*	handle = NULL ;
	va_list		ap ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

	if ((node1 = (*handle).searchNode(node_id)) == NULL) {
	    _sw_errno = SWE_NONODE ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}

	va_start(ap, g_attr) ;

	switch (g_attr) {	
	case GNODELABEL:
	    temp_str = va_arg(ap, LABEL) ;
	    strcpy(temp_str, (*node1).getLabel()) ;
	    break ;

	case GNODETYPE:
	    val = va_arg(ap, int*) ;
	    *val = (*node1).getType() ;
	    break ;

	case GNODECOLOR:
	    val = va_arg(ap, int*) ;
	    *val = (*node1).getColor() ;
	    break ;

	case GNODEFILLED:
	    val = va_arg(ap, int*) ;
	    *val = (*node1).getFilled() ;
	    break ;

	case GNODEWIDTH:
	    val = va_arg(ap, int*) ;
	    *val = (*node1).getWidth() ;
	    break ;

	case GNODEHEIGHT:
	    val = va_arg(ap, int*) ;
	    *val = (*node1).getHeight() ;
	    break ;

	case GNODELINETH:
	    val = va_arg(ap, int*) ;
	    *val = (*node1).getLineTH() ;
	    break ;

	case GNODEPOS:
	    pos = (*node1).getRPos() ;
	    val = va_arg(ap, int*) ;
	    *val = pos.x ;
	    val = va_arg(ap, int*) ;
	    *val = pos.y ;
	    break ;

	default:
	    break ;
	}
	return TRUE ;
}
	
bool sw_setgraphpos(GRAPHID g_id, int g_x, int g_y)
{
	BasicGraph	*handle = NULL ;
	LOCATION	temp_pos ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	
	temp_pos.x = g_x ;
	temp_pos.y = g_y ;
        handle->setPPos(temp_pos) ;
	return TRUE ;
}

bool sw_getgraphpos(GRAPHID g_id, int *g_x, int *g_y)
{
	BasicGraph	*handle = NULL ;
	LOCATION	temp_pos ;

	if ((handle = _sw_searchgraph(g_id)) == NULL) {
	    _sw_errno = SWE_NOGRAPH ;
	    _sw_geterrormsg(_sw_errno, _errorstr) ;
	    GSelog(0, "Swan:%s\n", _errorstr) ;
	    return FALSE ;
	}
	
        temp_pos = handle->getPPos() ;
	*g_x = temp_pos.x ;
	*g_y = temp_pos.y ;
	return TRUE ;
}
void sw_print(char* str)
{
	int	text_x, text_y ;
	int	width, height ;
	GSColor color ;

	if (str == NULL) 
	    OutputTextString[0] = '\0' ;
	else 
	    strcpy(OutputTextString, str) ;
	GSsetcurrentnamedwindow("root") ;
	GSgetfieldrect("MESSAGE", &text_x, &text_y, &width, &height, 
							&color) ;
	GSsetcolor(GSconvertcolor(color_table[DGRAY])) ;
	GSsetfont(LARGE) ;
	GSwfillrect(text_x, text_y, width, height) ;
	GSsetcolor(color) ;
	GSwwritemsg(text_x, text_y+12, OutputTextString) ;
	GSsetfont(SMALL) ;
	GSdoeventsandreturn() ;
}

void sw_getstr(char* str_ptr)
{
	fWaitStrInput = TRUE ;
	while (fWaitStrInput) 
	    GSdoeventsandreturn() ;
	strcpy(str_ptr, _sw_pick_string) ;
}

// Process Control Functions 

void sw_run(void)
{
	if (fProgState != RUN) {
	    fProgState = RUN ;
	    showstepframe(FALSE) ;
	    showrunframe(TRUE) ;
	}
}

void sw_step(void)
{
	if (fProgState != STEP) {
	    fProgState = STEP ;
	    showrunframe(FALSE) ;
	    showstepframe(TRUE) ;
	}
}

void sw_break(void)
{
	if (fProgState == STEP) 
	    sw_wait(STEP|RUN) ;
}

bool sw_pickgraph(GRAPHID* graph_p) 
{
	fWaitGraphPicked = TRUE ;
	while (fWaitGraphPicked) 
	    GSdoeventsandreturn() ;
	*graph_p = (*_sw_pick_graph).getID() ;
	return TRUE ;
}

bool sw_picknode(NODEID* node_p) 
{
	fWaitNodePicked = TRUE ;
	while (fWaitNodePicked) 
	    GSdoeventsandreturn() ;
	*node_p = (*_sw_pick_node).getID() ;
	return TRUE ;
}

bool sw_pickedge(NODEID* node1, NODEID* node2) 
{
	fWaitEdgePicked = TRUE ;
	while (fWaitEdgePicked) 
	    GSdoeventsandreturn() ;
	*node1 = (*(_sw_pick_edge->s_node)).getID() ;
	*node2 = (*(_sw_pick_edge->e_node)).getID() ;
	return TRUE ;
}

// suspicious: the definition of the position has to be clarified
bool sw_pickpos(int* x, int* y) 
{
	fWaitPosPicked = TRUE ;
	while (fWaitPosPicked) 
	    GSdoeventsandreturn() ;
	*x = _sw_pick_pos.x ;
	*y = _sw_pick_pos.y ;
	return TRUE ;
}

bool sw_getpickedgraph(GRAPHID* graph_p)
{
	if (_sw_pick_graph != NULL) {
	    *graph_p = (*_sw_pick_graph).getID() ;
	    return TRUE ;
	}
	else 
	    return FALSE ;
}

bool sw_getpickednode(NODEID* node_p)
{
	if (_sw_pick_node != NULL) {
	    *node_p = (*_sw_pick_node).getID() ;
	    return TRUE ;
	}
	return FALSE ;
}

bool sw_getpickededge(NODEID* node1, NODEID* node2)
{
	if (_sw_pick_edge != NULL) {
	    *node1 = (*(_sw_pick_edge->s_node)).getID() ;
	    *node2 = (*(_sw_pick_edge->e_node)).getID() ;
	    return TRUE ;
	}
	return FALSE ;
}

bool sw_getpickedpos(int* pos_x, int* pos_y)
{
	if (_sw_pick_pos != UnsetPos) {
	    *pos_x = _sw_pick_pos.x ;
	    *pos_y = _sw_pick_pos.y ;
	    return TRUE ;
	}
	return FALSE ;
}

// Error Processing 
//	. sw_errno
//	. sw_errmsg			

int sw_errno(void) 
{
	return _sw_errno ;
}

bool sw_errmsg(int errno, char* err_msg) 
{
	return _sw_geterrormsg(errno, err_msg) ;
}

/************************************************************
 * End of swintr.c					    *
 ************************************************************/
