/**********************************************************
 * swmain.c ---- Swan system internal functions           *
 * Jun Yang, November 17, 1994				  *
 **********************************************************/

/*--------------- Header Files ---------------*/

#include <iostream.h>
extern "C" {
#include "geosim.h"
}
#include "swan.h" 

/*--------------- Constants --------------*/

#define MINGRAPHDIST	50

/*--------------- Global Variables ---------------*/

GRAPHLIST        *ggl ; // Global Graph List
BasicLayoutComp  *_sw_root_lc = NULL ; // the root layout comp
INDEX		 _sw_errno = 0 ; 
NODETYPE	 _sw_cur_node_type = BOX ;

// current global graphic attributes
FLAG             _sw_cur_fDirected ;
NODETYPE         _sw_cur_nodeType ;
COLORTYPE        _sw_cur_nodeColor ;
FLAG             _sw_cur_nodeFilled ;
int              _sw_cur_nodeWidth ;
int              _sw_cur_nodeHeight ;
int              _sw_cur_nodeLineTH ;
int              _sw_cur_edgeLength ;
int              _sw_cur_edgeLineTH ;
COLORTYPE        _sw_cur_edgeColor ;
int		 _sw_nodepicked ;

// Swan system switches
FLAG		 _sws_autoredraw ;
FLAG		 _sws_swapnode ;
FLAG		 _sws_movenode ;
FLAG		 _sws_insertnode ;
FLAG		 _sws_deletenode ;
FLAG		 _sws_insertedge ;
FLAG		 _sws_deleteedge ;

// Swan internal global variables
FLAG		fSwanInited = FALSE ;
FLAG		fSwanWinInited = FALSE ;

// Used by sw_enablemenu and sw_disablemenu to convert menu and item
// numbers to names
char *SW_MENU[] = {"edit_menu"};
char *SW_MENUITEM[] = {"InsNode", "DelNode", "InsEdge", "DelEdge"};

/*--------------- Swan kernel functions ---------------*/

bool _sw_init()
{

	ggl = new GRAPHLIST ;
	_sw_root_lc = new BasicLayoutComp(NULLLCID, LISTACROSS) ;
	(*_sw_root_lc).setPID(NULL) ;

        _sw_cur_fDirected = DIRECTED ;
        _sw_cur_nodeType = BOX ;
        _sw_cur_nodeColor = DGREEN ;
        _sw_cur_nodeFilled = FALSE ;
        _sw_cur_nodeWidth = RECTNODEWIDTH ;
        _sw_cur_nodeHeight = RECTNODEHEIGHT ;
        _sw_cur_nodeLineTH = THINLINE ;
        _sw_cur_edgeLength = 0 ;
        _sw_cur_edgeLineTH = THINLINE ;
        _sw_cur_edgeColor = BLACK ;
	_sw_nodepicked = 0 ;
	// set Swan switches
	_sws_autoredraw = ON ;
	_sws_swapnode = OFF ;
	_sws_movenode = OFF ;
	_sws_insertnode = OFF ;
	_sws_deletenode = OFF ;
	_sws_insertedge = OFF ;
	_sws_deleteedge = OFF ;
	
        sw_disablemenuitem(EDITMENU, ITEMINSNODE) ;
        sw_disablemenuitem(EDITMENU, ITEMDELNODE) ;
        sw_disablemenuitem(EDITMENU, ITEMINSEDGE) ;
        sw_disablemenuitem(EDITMENU, ITEMDELEDGE) ;

	GSdisablebutton("root", "SwapNode", DRAW) ;
	GSdisablebutton("root", "MoveNode", DRAW) ;

	fProgState = RUN ;
	return TRUE ; 
}

bool _sw_quit()
{
	BasicGraph*	tempGraph = NULL ;
	bool		ok ;

	if (ggl != NULL) {
	    ok = (*ggl).getFirstNode(tempGraph) ;
	    while (ok) {
	        delete tempGraph ;
	        ok = (*ggl).getNextNode(tempGraph) ;
	    } 
	    delete ggl ;
	    ggl = NULL ;
	}

	if (_sw_root_lc != NULL)  {
	    delete _sw_root_lc ;
	    _sw_root_lc = NULL ;
	}
	return TRUE ;
}

void _sw_set_allgraphs_pos(void)
{
	LOCATION	temp_pos ;
	BasicGraph*	temp_graph ;
	BasicLayoutComp* temp_lc ;
	bool		ok ;
	BOXSIZE		temp_size ;

	temp_pos.x = temp_pos.y = 10 ;
	ok = (*ggl).getFirstNode(temp_graph) ;
	while (ok) {
	    if ((*temp_graph).getLayoutMode() == AUTOLAYOUT) {
	        temp_lc = (*temp_graph).getRootLC() ;
	        temp_size = (*temp_lc).getBoxSize() ;
	        (*temp_lc).setPPos(temp_pos) ;
	        temp_pos.x += temp_size.w + MINGRAPHDIST ;
	    }
	    ok = (*ggl).getNextNode(temp_graph) ;
	}
}

bool _sw_normalizeRPos(BasicGraph* g_ptr)
{
	NODELIST	*temp_nodelist ;
	BasicNode	*temp_node ;
	bool		ok ;
	LOCATION	temp_pos, pos1 ;
	BOXSIZE		temp_size ;
	int		offset_x, offset_y ;

	temp_pos.x = MAX_XCOORD ;
	temp_pos.y = MAX_YCOORD ;
	temp_nodelist = (*g_ptr).getNodeList() ;
	ok = (*temp_nodelist).getFirstNode(temp_node) ;
	while (ok) {
	    pos1 = (*temp_node).getRPos() ;
	    if (pos1.x < temp_pos.x)
		temp_pos.x = pos1.x ;
	    if (pos1.y < temp_pos.y)
		temp_pos.y = pos1.y ;
	    ok = (*temp_nodelist).getNextNode(temp_node) ;
	}
	offset_x = MINLCMARGIN - temp_pos.x ;
	offset_y = MINLCMARGIN - temp_pos.y ;
	ok = (*temp_nodelist).getFirstNode(temp_node) ;
	while (ok) {
	    pos1 = (*temp_node).getRPos() ;
	    pos1.x += offset_x ;
	    pos1.y += offset_y ;
	    (*temp_node).setRPos(pos1) ;
	    ok = (*temp_nodelist).getNextNode(temp_node) ;
	}
	return TRUE ;
}

BasicGraph* _sw_searchgraph(GRAPHID g_id)
{
	BasicGraph*	handle = NULL ;
	bool		ok ;

	ok = (*ggl).getFirstNode(handle) ;
	while (ok) {
	    if ((*handle).getID() == g_id) 
		return handle ;
	    ok = (*ggl).getNextNode(handle) ;
	}
	return NULL ;
}

bool _sw_segedgepos(LOCATION spos, LOCATION epos, 
		       BOXSIZE lsize, EDGEINFO*& lptr)
{
        long a1, a2, b1, b2, c1, c2 ; /* Coefficients of line eqns. */
        long denom, offset, num ;     /* Intermediate values */
	LOCATION apos, cpos ;
	bool	sflag ;

	// the line segment is horizontal
	if (spos.y == epos.y) {
	    lptr->label_ppos.y = spos.y - 1 ;
	    lptr->label_ppos.x = (spos.x + epos.x - lsize.w) / 2 ;
	    lptr->label_orient = HORIZONTAL ;
	    return TRUE ;
	}

	// the line segment is vertical
	if (spos.x == epos.x) {
	    lptr->label_ppos.y = (spos.y + epos.y) / 2 ;
	    lptr->label_ppos.x = spos.x - 1 - lsize.w ;
	    lptr->label_orient = VERTICAL ;
	    return TRUE ;
	}
	
	// general situations
	if (spos.x < epos.x) 
	    sflag = spos.y < epos.y ? FALSE : TRUE ;
	else  // spos.x > epos.x 
	    sflag = spos.y < epos.y ? TRUE : FALSE ;
	
	// get the mid-point 
	apos.x = (spos.x + epos.x) / 2 ;
	apos.y = (spos.y + epos.y) / 2 ;

	// set coefficients for first equation

        a1 = epos.y - spos.y ;
        b1 = spos.x - epos.x ;
        c1 = epos.x * spos.y - spos.x * epos.y ;

	// set coefficients for the second equation
	
	a2 = epos.x - spos.x ;
	b2 = epos.y - spos.y ;
	if (sflag) 
	    c2 = a2 * (-lsize.w / 2 - apos.x) +
	         b2 * (-lsize.h / 2 - apos.y) ;
	else 
	    c2 = a2 * (-lsize.w / 2 - apos.x) +
	         b2 * (lsize.h / 2 - apos.y) ;

	// solve the equations
        denom = a1 * b2 - a2 * b1 ;
        offset = denom < 0 ? - denom / 2 : denom /2 ;

        /* The denom/2 is to get rounding instead of truncating. It
         * is added or subtracted to the numerator, depending upon the
         * sign of the numerator.
         */

        num = b1 * c2 - b2 * c1 ;
        cpos.x = ( num < 0 ? num - offset : num + offset) / denom ;

        num = a2 * c1 - a1 * c2 ;
        cpos.y = ( num < 0 ? num - offset : num + offset) / denom ;
	
	if (sflag) {
	    lptr->label_ppos.x = cpos.x - lsize.w ;
	    lptr->label_ppos.y = cpos.y ;
	    lptr->label_orient = VICEDIAGONAL ;
	}
	else {
	    lptr->label_ppos.x = cpos.x - lsize.w ;
	    lptr->label_ppos.y = cpos.y + lsize.h ;
	    lptr->label_orient = DIAGONAL ;
	}
	return TRUE ;	
}

// Swan handler of new exception 
void out_of_store(void)
{
	GSelog(SWE_NOMEMORY, "Swan: out of memory!\n") ;
}

// Buttons controller
void _sw_check_buttons(void)
{
	BasicGraph*	temp_g ;
	BasicLayoutComp *root_lc, *temp_lc ;
	bool		fSwap = FALSE ;
	bool		fMove = FALSE ;
	bool		ok ;

	if (ggl == NULL) 
	    return ;
	
	ok = (*ggl).getFirstNode(temp_g) ;
	while (ok) {
	    root_lc = (*temp_g).getRootLC() ;
	    if (root_lc->getChLCList()->getFirstNode(temp_lc))
	 	switch ((*temp_lc).getType()) {
		case KKNET:
		    fSwap = TRUE ;
		    fMove = TRUE ;
		    break ;
		case CIRCLENET:
		    fSwap = TRUE ;
		    break ;
		default:
		    break ;
		}
	    ok = (*ggl).getNextNode(temp_g) ;
	}
	if (fSwap) 
	    GSenablebutton("root", "SwapNode", DRAW) ;
	else
	    GSdisablebutton("root", "SwapNode", DRAW) ;
	if (fMove) 
	    GSenablebutton("root", "MoveNode", DRAW) ;
	else
	    GSdisablebutton("root", "MoveNode", DRAW) ;
}

	    
/**********************************************************
 *  End of swmain.c				          *
 **********************************************************/
